@@ -7,19 +7,21 @@
  ***************************************************************************/
 
 /*
- *  This program is free software; you can redistribute it and/or modify
+ *  Brasero is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Brasero is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 #include <stdlib.h>
@@ -546,11 +548,6 @@ brasero_burn_dialog_insert_disc_cb (BraseroBurn *burn,
 
 	g_free (main_message);
 
-	if (error == BRASERO_BURN_ERROR_MEDIA_NONE)
-		gtk_window_set_title (GTK_WINDOW (message), _("Disc Insertion"));
-	else
-		gtk_window_set_title (GTK_WINDOW (message), _("Disc Replacement"));
-
 	/* connect to signals to be warned when media is inserted */
 	added_id = g_signal_connect_after (drive,
 					   "medium-added",
@@ -565,7 +562,6 @@ brasero_burn_dialog_insert_disc_cb (BraseroBurn *burn,
 	/* see if we should update the infos */
 	medium = brasero_burn_session_get_src_medium (dialog->priv->session);
 	if (dialog->priv->input.type == BRASERO_TRACK_TYPE_DISC) {
-		BraseroMedium *medium;
 		BraseroDrive *src;
 
 		/* see if the drive is the source */
