@@ -3,12 +3,10 @@
  * brasero
  * Copyright (C) Philippe Rouquier 2007-2008 <bonfire-app@wanadoo.fr>
  * 
- * brasero is free software.
- * 
- * You may redistribute it and/or modify it under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option)
- * any later version.
+ *  Brasero is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  * 
  * brasero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -1294,7 +1292,8 @@ brasero_data_disc_set_session_contents (BraseroDisc *self,
 	/* Set the number of files in the tree */
 	root = brasero_data_project_get_root (priv->project);
 	stats = BRASERO_FILE_NODE_STATS (root);
-	brasero_track_set_data_file_num (track, stats->children);
+	if (stats)
+		brasero_track_set_data_file_num (track, stats->children);
 
 	joliet_compat = (type.subtype.fs_type & BRASERO_IMAGE_FS_JOLIET);
 	brasero_track_add_data_fs (track, type.subtype.fs_type);
