@@ -3,12 +3,10 @@
  * brasero
  * Copyright (C) Philippe Rouquier 2007-2008 <bonfire-app@wanadoo.fr>
  * 
- * brasero is free software.
- * 
- * You may redistribute it and/or modify it under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option)
- * any later version.
+ *  Brasero is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  * 
  * brasero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -1298,6 +1296,7 @@ brasero_data_project_move_node (BraseroDataProject *self,
 	/* unparent node now in case its target sibling is a parent */
 	former_parent = node->parent;
 	former_position = brasero_file_node_get_pos_as_child (node);
+	stats = brasero_file_node_get_tree_stats (priv->root, NULL);
 	brasero_file_node_move_from (node, stats);
 
 	klass = BRASERO_DATA_PROJECT_GET_CLASS (self);
