@@ -3,12 +3,10 @@
  * brasero
  * Copyright (C) Philippe Rouquier 2007-2008 <bonfire-app@wanadoo.fr>
  *
- * brasero is free software.
- *
- * You may redistribute it and/or modify it under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option)
- * any later version.
+ *  Brasero is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  *
  * brasero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -302,6 +300,37 @@ brasero_dest_selection_get_output_path (BraseroDestSelection *self)
 	return path;
 }
 
+static void
+brasero_dest_selection_set_output_path (BraseroDestSelection *self,
+					BraseroImageFormat format,
+					const gchar *path)
+{
+	BraseroDestSelectionPrivate *priv;
+
+	priv = BRASERO_DEST_SELECTION_PRIVATE (self);
+
+	switch (format) {
+	case BRASERO_IMAGE_FORMAT_BIN:
+		brasero_burn_session_set_image_output_full (priv->session,
+							    format,
+							    path,
+							    NULL);
+		break;
+
+	case BRASERO_IMAGE_FORMAT_CDRDAO:
+	case BRASERO_IMAGE_FORMAT_CLONE:
+	case BRASERO_IMAGE_FORMAT_CUE:
+		brasero_burn_session_set_image_output_full (priv->session,
+							    format,
+							    NULL,
+							    path);
+		break;
+
+	default:
+		break;
+	}
+}
+
 static void
 brasero_dest_selection_get_default_output_format (BraseroDestSelection *self,
 						  BraseroTrackType *output)
@@ -326,8 +355,24 @@ brasero_dest_selection_get_default_output_format (BraseroDestSelection *self,
 		return;
 	}
 
-	if (source.type == BRASERO_TRACK_TYPE_AUDIO)
+	if (source.type == BRASERO_TRACK_TYPE_AUDIO) {
+		/* If that's AUDIO only without VIDEO then return */
+		if (!(source.subtype.audio_format & (BRASERO_VIDEO_FORMAT_UNDEFINED|BRASERO_VIDEO_FORMAT_VCD|BRASERO_VIDEO_FORMAT_VIDEO_DVD)))
+			return;
+
+		/* Otherwise try all possible image types */
+		output->subtype.img_format = BRASERO_IMAGE_FORMAT_CDRDAO;
+		for (; output->subtype.img_format != BRASERO_IMAGE_FORMAT_NONE;
+		       output->subtype.img_format >>= 1) {
+		
+			result = brasero_burn_caps_is_output_supported (priv->caps,
+									priv->session,
+									output);
+			if (result == BRASERO_BURN_OK)
+				return;
+		}
 		return;
+	}
 
 	if (source.type == BRASERO_TRACK_TYPE_DATA
 	||  source.subtype.media & (BRASERO_MEDIUM_DVD|BRASERO_MEDIUM_DVD_DL)) {
@@ -505,6 +550,9 @@ brasero_dest_selection_image_format_changed_cb (BraseroImageProperties *dialog,
 
 	/* make sure the extension is still valid */
 	image_path = brasero_image_properties_get_path (dialog);
+	if (!image_path)
+		return;
+
 	format = brasero_image_properties_get_format (dialog);
 
 	if (format == BRASERO_IMAGE_FORMAT_ANY || format == BRASERO_IMAGE_FORMAT_NONE) {
@@ -519,8 +567,10 @@ brasero_dest_selection_image_format_changed_cb (BraseroImageProperties *dialog,
 		g_free (image_path);
 		image_path = brasero_dest_selection_get_default_output_path (self, format);
 	}
-	else
+	else if (image_path)
 		image_path = brasero_dest_selection_fix_image_extension (format, FALSE, image_path);
+	else
+		image_path = brasero_dest_selection_get_default_output_path (self, format);
 
 	brasero_image_properties_set_path (dialog, image_path);
 }
@@ -668,7 +718,7 @@ brasero_dest_selection_image_properties (BraseroDestSelection *self)
 		image_path = brasero_image_properties_get_path (BRASERO_IMAGE_PROPERTIES (priv->drive_prop));
 
 		if (!brasero_dest_selection_image_check_extension (self, format, image_path)) {
-			if (!brasero_dest_selection_image_extension_ask (self)) {
+			if (brasero_dest_selection_image_extension_ask (self)) {
 				priv->default_ext = TRUE;
 				image_path = brasero_dest_selection_fix_image_extension (format, TRUE, image_path);
 			}
@@ -683,9 +733,9 @@ brasero_dest_selection_image_properties (BraseroDestSelection *self)
 	priv->drive_prop = NULL;
 
 	brasero_drive_selection_set_image_path (BRASERO_DRIVE_SELECTION (self), image_path);
-	brasero_burn_session_set_image_output (priv->session,
-					       format,
-					       image_path);
+	brasero_dest_selection_set_output_path (self,
+						format,
+						image_path);
 	g_free (image_path);
 }
 
@@ -996,9 +1046,10 @@ brasero_dest_selection_set_image_properties (BraseroDestSelection *self)
 	||  output.subtype.img_format == BRASERO_IMAGE_FORMAT_NONE) {
 		/* That means that we've got a problem */
 		/* FIXME: we need to display a message nevertheless */
-		brasero_burn_session_set_image_output (priv->session,
-						       BRASERO_IMAGE_FORMAT_NONE,
-						       NULL);
+		brasero_burn_session_set_image_output_full (priv->session,
+							    BRASERO_IMAGE_FORMAT_NONE,
+							    NULL,
+							    NULL);
 		g_signal_emit (self,
 			       brasero_dest_selection_signals [VALID_MEDIA_SIGNAL],
 			       0,
@@ -1014,11 +1065,9 @@ brasero_dest_selection_set_image_properties (BraseroDestSelection *self)
 		       0,
 		       TRUE);
 	gtk_widget_set_sensitive (priv->button, TRUE);
-
-	brasero_burn_session_set_image_output (priv->session,
-					       output.subtype.img_format,
-					       path);
-
+	brasero_dest_selection_set_output_path (self,
+						output.subtype.img_format,
+						path);
 	brasero_drive_selection_set_image_path (BRASERO_DRIVE_SELECTION (self),
 						path);
 	g_free (path);
@@ -1090,7 +1139,9 @@ brasero_dest_selection_check_image_settings (BraseroDestSelection *self)
 		else
 			path = brasero_dest_selection_get_default_output_path (self, format);
 
-		brasero_burn_session_set_image_output (priv->session, format, path);
+		brasero_dest_selection_set_output_path (self,
+							format,
+							path);
 		brasero_drive_selection_set_image_path (BRASERO_DRIVE_SELECTION (self), path);
 		g_free (path);
 	}
