@@ -3,12 +3,10 @@
  * brasero
  * Copyright (C) Philippe Rouquier 2007-2008 <bonfire-app@wanadoo.fr>
  * 
- * brasero is free software.
- * 
- * You may redistribute it and/or modify it under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option)
- * any later version.
+ *  Brasero is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  * 
  * brasero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,6 +24,8 @@
 #  include <config.h>
 #endif
 
+#include <string.h>
+
 #include <glib.h>
 #include <glib-object.h>
 #include <glib/gi18n-lib.h>
@@ -61,6 +61,10 @@ struct _BraseroDrivePropertiesPrivate
 
 	GtkWidget *tmpdir;
 	GtkWidget *tmpdir_size;
+
+	gchar *previous_path;
+
+	guint check_filesystem:1;
 };
 
 #define BRASERO_DRIVE_PROPERTIES_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), BRASERO_TYPE_DRIVE_PROPERTIES, BraseroDrivePropertiesPrivate))
@@ -129,40 +133,32 @@ brasero_drive_properties_get_tmpdir (BraseroDriveProperties *self)
 	return gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (priv->tmpdir));
 }
 
-void
-brasero_drive_properties_set_tmpdir (BraseroDriveProperties *self,
-				     const gchar *path)
+static gboolean
+brasero_drive_properties_set_tmpdir_info (BraseroDriveProperties *self,
+					  const gchar *path,
+					  gboolean warn)
 {
 	GFile *file;
 	gchar *string;
 	GFileInfo *info;
-	gchar *directory;
 	GError *error = NULL;
 	guint64 vol_size = 0;
+	const gchar *filesystem;
 	BraseroDrivePropertiesPrivate *priv;
 
 	priv = BRASERO_DRIVE_PROPERTIES_PRIVATE (self);
 
-	if (!path)
-		gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (priv->tmpdir),
-					       g_get_tmp_dir ());
-	else
-		gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (priv->tmpdir),
-					       path);
-
 	/* get the volume free space */
-	directory = g_path_get_dirname (path);
-	file = g_file_new_for_commandline_arg (directory);
-	g_free (directory);
-
-	if (file == NULL) {
+	file = g_file_new_for_commandline_arg (path);
+	if (!file) {
 		BRASERO_BURN_LOG ("impossible to retrieve size for %s", path);
 		gtk_label_set_text (GTK_LABEL (priv->tmpdir_size), _("unknown"));
-		return;
+		return FALSE;
 	}
 
 	info = g_file_query_filesystem_info (file,
-					     G_FILE_ATTRIBUTE_FILESYSTEM_FREE,
+					     G_FILE_ATTRIBUTE_FILESYSTEM_FREE ","
+					     G_FILE_ATTRIBUTE_FILESYSTEM_TYPE,
 					     NULL,
 					     &error);
 	g_object_unref (file);
@@ -174,7 +170,51 @@ brasero_drive_properties_set_tmpdir (BraseroDriveProperties *self,
 		g_error_free (error);
 
 		gtk_label_set_text (GTK_LABEL (priv->tmpdir_size), _("unknown"));
-		return;
+		return FALSE;
+	}
+
+	/* NOTE/FIXME: also check, probably best at start or in a special dialog
+	 * whether quotas or any other limitation enforced on the system may not
+	 * get in out way. Think getrlimit (). */
+
+	/* check the filesystem type: the problem here is that some
+	 * filesystems have a maximum file size limit of 4 Gio and more than
+	 * often we need a temporary file size of 4 Gio or more. */
+	filesystem = g_file_info_get_attribute_string (info, G_FILE_ATTRIBUTE_FILESYSTEM_TYPE);
+	if (priv->check_filesystem
+	&&  filesystem
+	&& !strcmp (filesystem, "msdos")) {
+		gint answer;
+		GtkWidget *dialog;
+		GtkWidget *toplevel;
+
+		toplevel = gtk_widget_get_toplevel (GTK_WIDGET (self));
+		dialog = gtk_message_dialog_new (GTK_WINDOW (toplevel),
+						 GTK_DIALOG_DESTROY_WITH_PARENT |
+						 GTK_DIALOG_MODAL,
+						 GTK_MESSAGE_WARNING,
+						 GTK_BUTTONS_NONE,
+						 _("Do you really want to choose this location?"));
+
+		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+							  _("The filesystem on this volume doesn't support large files (size over 2 Gio)."
+							    "\nThis can be a problem when writing DVDs or large images."));
+
+		gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+					_("_Keep current location"), GTK_RESPONSE_CANCEL,
+					_("_Change location"), GTK_RESPONSE_OK,
+					NULL);
+
+		gtk_widget_show_all (dialog);
+		answer = gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog);
+
+		if (answer != GTK_RESPONSE_OK) {
+			g_object_unref (info);
+			return FALSE;
+		}
+
+		priv->check_filesystem = 1;
 	}
 
 	vol_size = g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_FILESYSTEM_FREE);
@@ -183,6 +223,72 @@ brasero_drive_properties_set_tmpdir (BraseroDriveProperties *self,
 	string = brasero_utils_get_size_string (vol_size, TRUE, TRUE);
 	gtk_label_set_text (GTK_LABEL (priv->tmpdir_size), string);
 	g_free (string);
+
+	return TRUE;
+}
+
+static void
+brasero_drive_properties_tmpdir_changed_cb (GtkFileChooser *chooser,
+					    BraseroDriveProperties *self)
+{
+	gchar *path;
+	gboolean result;
+	BraseroDrivePropertiesPrivate *priv;
+
+	priv = BRASERO_DRIVE_PROPERTIES_PRIVATE (self);
+
+	priv->check_filesystem = 1;
+	path = gtk_file_chooser_get_filename (chooser);
+	result = brasero_drive_properties_set_tmpdir_info (self, path, TRUE);
+	g_free (path);
+
+	if (result) {
+		if (priv->previous_path)
+			g_free (priv->previous_path);
+
+		priv->previous_path = gtk_file_chooser_get_filename (chooser);
+		return;
+	}
+
+	g_signal_handlers_block_by_func (chooser,
+					 brasero_drive_properties_tmpdir_changed_cb,
+					 self);
+	if (priv->previous_path)
+		gtk_file_chooser_set_filename (chooser, priv->previous_path);
+	else
+		gtk_file_chooser_set_filename (chooser, g_get_tmp_dir ());
+
+	g_signal_handlers_unblock_by_func (chooser,
+					   brasero_drive_properties_tmpdir_changed_cb,
+					   self);
+}
+
+void
+brasero_drive_properties_set_tmpdir (BraseroDriveProperties *self,
+				     const gchar *path)
+{
+	BraseroDrivePropertiesPrivate *priv;
+
+	priv = BRASERO_DRIVE_PROPERTIES_PRIVATE (self);
+
+	if (!path)
+		path = g_get_tmp_dir ();
+
+	priv->check_filesystem = 0;
+	brasero_drive_properties_set_tmpdir_info (self, path, FALSE);
+
+	g_signal_handlers_block_by_func (GTK_FILE_CHOOSER (priv->tmpdir),
+					 brasero_drive_properties_tmpdir_changed_cb,
+					 self);
+	gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (priv->tmpdir), path);
+	g_signal_handlers_unblock_by_func (GTK_FILE_CHOOSER (priv->tmpdir),
+					   brasero_drive_properties_tmpdir_changed_cb,
+					   self);
+
+	if (priv->previous_path)
+		g_free (priv->previous_path);
+
+	priv->previous_path = g_strdup (path);
 }
 
 static void
@@ -411,12 +517,26 @@ brasero_drive_properties_init (BraseroDriveProperties *object)
 	gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (priv->tmpdir),
 				       g_get_tmp_dir ());
 
+	g_signal_connect (priv->tmpdir,
+			  "file-set",
+			  G_CALLBACK (brasero_drive_properties_tmpdir_changed_cb),
+			  object);
+
 	gtk_widget_show_all (GTK_DIALOG (object)->vbox);
 }
 
 static void
 brasero_drive_properties_finalize (GObject *object)
 {
+	BraseroDrivePropertiesPrivate *priv;
+
+	priv = BRASERO_DRIVE_PROPERTIES_PRIVATE (object);
+
+	if (priv->previous_path) {
+		g_free (priv->previous_path);
+		priv->previous_path = NULL;
+	}
+
 	G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
