@@ -7,12 +7,12 @@
  ****************************************************************************/
 
 /*
- * This program is free software; you can redistribute it and/or modify
+ * Brasero is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  * 
- * This program is distributed in the hope that it will be useful,
+ * Brasero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU Library General Public License for more details.
@@ -690,7 +690,6 @@ brasero_file_node_add (BraseroFileNode *parent,
 
 	node->parent = parent;
 	if (!node->is_imported) {
-		BraseroFileTreeStats *stats;
 		guint depth = 0;
 
 		if (!node->is_grafted) {
@@ -702,12 +701,6 @@ brasero_file_node_add (BraseroFileNode *parent,
 			}
 		}
 
-		stats = brasero_file_node_get_tree_stats (parent, &depth);
-		if (node->is_file) {
-			/* only count files */
-			stats->children ++;
-		}
-
 		if (depth > 6)
 			node->is_deep = TRUE;
 	}
@@ -723,6 +716,15 @@ brasero_file_node_set_from_info (BraseroFileNode *node,
 	 * creation of a graft). If someone wants to set a new name,
 	 * then rename_node is the function. */
 
+	/* update the stats since a file could have been added to the tree but
+	 * at this point we didn't know what it was (a file or a directory).
+	 * Only do this if it wasn't a file before. */
+	if (!node->is_file
+	&& (g_file_info_get_file_type (info) != G_FILE_TYPE_DIRECTORY)) {
+		/* only count files */
+		stats->children ++;
+	}
+
 	/* update :
 	 * - the mime type
 	 * - the size (and possibly the one of his parent)
