@@ -3,12 +3,10 @@
  * brasero
  * Copyright (C) Philippe Rouquier 2007-2008 <bonfire-app@wanadoo.fr>
  * 
- * brasero is free software.
- * 
- * You may redistribute it and/or modify it under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option)
- * any later version.
+ *  Brasero is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  * 
  * brasero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -100,6 +98,12 @@ brasero_image_properties_is_path_edited (BraseroImageProperties *self)
 		return TRUE;
 
 	chooser_path = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (self));
+	if (!chooser_path && priv->original_path)
+		return TRUE;
+
+	if (!priv->original_path && chooser_path)
+		return TRUE;
+
 	if (!strcmp (chooser_path, priv->original_path))
 		return FALSE;
 
@@ -129,7 +133,12 @@ brasero_image_properties_set_path (BraseroImageProperties *self,
 
 			/* check if the path was edited since the last time it was set */
 			chooser_path = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (self));
-			priv->edited = strcmp (priv->original_path, chooser_path) != 0;
+			priv->edited = 	(!chooser_path && priv->original_path) ||
+				        (!priv->original_path && chooser_path) ||
+					 strcmp (priv->original_path, chooser_path) != 0;
+
+			if (chooser_path)
+				g_free (chooser_path);
 		}
 		g_free (priv->original_path);
 	}
