@@ -1,14 +1,12 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /*
  * brasero
- * Copyright (C) Philippe Rouquier 2008 <bonfire-app@wanadoo.fr>
+ * Copyright (C) Philippe Rouquier 2005-2008 <bonfire-app@wanadoo.fr>
  * 
- * brasero is free software.
- * 
- * You may redistribute it and/or modify it under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option)
- * any later version.
+ *  Brasero is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  * 
  * brasero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -1025,7 +1023,7 @@ brasero_jacket_view_update_color (BraseroJacketView *self)
 							       0.0,
 							       width,
 							       0.0);
-		else if (priv->color_style == BRASERO_JACKET_COLOR_VGRADIENT)
+		else /* if (priv->color_style == BRASERO_JACKET_COLOR_VGRADIENT) */
 			pattern = cairo_pattern_create_linear (0.0,
 							       0.0,
 							       0.0,
