@@ -7,19 +7,21 @@
 ****************************************************************************/
 
 /*
- *  This program is free software; you can redistribute it and/or modify
+ *  Brasero is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Brasero is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -50,7 +52,7 @@
 #include "brasero-player-bacon.h"
 #include "brasero-utils.h"
 #include "brasero-metadata.h"
-#include "brasero-vfs.h"
+#include "brasero-io.h"
 #include "burn-debug.h"
 
 G_DEFINE_TYPE (BraseroPlayer, brasero_player, GTK_TYPE_ALIGNMENT);
@@ -86,8 +88,8 @@ struct BraseroPlayerPrivate {
 
 	BraseroPlayerBaconState state;
 
-	BraseroVFS *vfs;
-	BraseroVFSDataID meta_task;
+	BraseroIO *io;
+	BraseroIOJobBase *meta_task;
 
 	gchar *uri;
 	gint64 start;
@@ -405,14 +407,27 @@ brasero_player_create_controls_stream (BraseroPlayer *player,
 			    0);
 
 	player->priv->size = gtk_label_new (NULL);
-	gtk_label_set_justify (GTK_LABEL (player->priv->size), GTK_JUSTIFY_RIGHT);
-	gtk_misc_set_alignment (GTK_MISC (player->priv->size), 1.0, 0.0);
-	gtk_box_pack_start (GTK_BOX (header_box),
-			    player->priv->size,
-			    FALSE,
-			    FALSE,
-			    0);
+	if (GTK_WIDGET (player)->allocation.width > GTK_WIDGET (player)->allocation.height) {
+		gtk_label_set_justify (GTK_LABEL (player->priv->size), GTK_JUSTIFY_RIGHT);
+		gtk_misc_set_alignment (GTK_MISC (player->priv->size), 1.0, 0.0);
 
+		gtk_box_pack_start (GTK_BOX (header_box),
+				    player->priv->size,
+				    FALSE,
+				    FALSE,
+				    0);
+	}
+	else {
+		gtk_label_set_justify (GTK_LABEL (player->priv->size), GTK_JUSTIFY_LEFT);
+		gtk_misc_set_alignment (GTK_MISC (player->priv->size), 0.0, 0.0);
+
+		gtk_box_pack_start (GTK_BOX (player->priv->controls),
+				    player->priv->size,
+				    FALSE,
+				    FALSE,
+				    0);
+	}
+	
 	/* second line : play, progress, volume button */
 	box = gtk_hbox_new (FALSE, 12);
 	gtk_box_pack_start (GTK_BOX (player->priv->controls),
@@ -440,6 +455,7 @@ brasero_player_create_controls_stream (BraseroPlayer *player,
 	player->priv->progress = gtk_hscale_new_with_range (0, 1, 500000000);
 	gtk_scale_set_digits (GTK_SCALE (player->priv->progress), 0);
 	gtk_scale_set_draw_value (GTK_SCALE (player->priv->progress), FALSE);
+	gtk_widget_set_size_request (player->priv->progress, 80, -1);
 	gtk_range_set_update_policy (GTK_RANGE (player->priv->progress), GTK_UPDATE_CONTINUOUS);
 	gtk_box_pack_start (GTK_BOX (box),
 			  player->priv->progress,
@@ -664,11 +680,18 @@ brasero_player_create_controls_image (BraseroPlayer *player)
 
 	player->priv->controls = gtk_vbox_new (FALSE, 4);
 
-	gtk_box_pack_end (GTK_BOX (player->priv->hbox),
-			  player->priv->controls,
-			  TRUE,
-			  TRUE,
-			  0);
+	if (GTK_WIDGET (player)->allocation.width > GTK_WIDGET (player)->allocation.height)
+		gtk_box_pack_end (GTK_BOX (player->priv->hbox),
+				  player->priv->controls,
+				  TRUE,
+				  TRUE,
+				  0);
+	else
+		gtk_box_pack_end (GTK_BOX (player->priv->vbox),
+				  player->priv->controls,
+				  TRUE,
+				  TRUE,
+				  0);
 
 	player->priv->header = gtk_label_new (NULL);
 	gtk_misc_set_alignment (GTK_MISC (player->priv->header), 0.0, 0.5);
@@ -745,7 +768,7 @@ brasero_player_image (BraseroPlayer *player)
 
 	/* image */
 	/* FIXME: this does not allow to preview remote files */
-	path = gnome_vfs_get_local_path_from_uri (player->priv->uri);
+	path = g_filename_from_uri (player->priv->uri, NULL, NULL);
 	player->priv->pixbuf = gdk_pixbuf_new_from_file (path, &error);
 
 	if (!player->priv->pixbuf) {
@@ -841,12 +864,10 @@ brasero_player_update_info_real (BraseroPlayer *player,
 }
 
 static void
-brasero_player_metadata_completed (BraseroVFS *vfs,
-				   GObject *obj,
-				   GnomeVFSResult result,
+brasero_player_metadata_completed (GObject *obj,
+				   GError *error,
 				   const gchar *uri,
-				   GnomeVFSFileInfo *info,
-				   BraseroMetadataInfo *metadata,
+				   GFileInfo *info,
 				   gpointer null_data)
 {
 	BraseroPlayer *player = BRASERO_PLAYER (obj);
@@ -860,7 +881,7 @@ brasero_player_metadata_completed (BraseroVFS *vfs,
 	if (player->priv->controls)
 		brasero_player_destroy_controls (player);
 
-	if (result != GNOME_VFS_OK) {
+	if (error) {
 		brasero_player_no_multimedia_stream (player);
 		g_signal_emit (player,
 			       brasero_player_signals [ERROR_SIGNAL],
@@ -869,12 +890,12 @@ brasero_player_metadata_completed (BraseroVFS *vfs,
 	}
 
 	/* based on the mime type, we try to determine the type of file */
-	if (metadata && metadata->has_video) {
+	if (g_file_info_get_attribute_boolean (info, BRASERO_IO_HAS_VIDEO)) {
 		/* video */
 		brasero_player_create_controls_stream (player, TRUE);
 		gtk_range_set_value (GTK_RANGE (player->priv->progress), 0.0);
 
-		if (metadata->is_seekable)
+		if (g_file_info_get_attribute_boolean (info, BRASERO_IO_IS_SEEKABLE))
 			gtk_widget_set_sensitive (player->priv->progress, TRUE);
 		else
 			gtk_widget_set_sensitive (player->priv->progress, FALSE);
@@ -883,18 +904,18 @@ brasero_player_metadata_completed (BraseroVFS *vfs,
 		gtk_widget_hide (player->priv->image_display);
 		gtk_widget_show (player->priv->notebook);
 	}
-	else if (metadata && metadata->has_audio) {
+	else if (g_file_info_get_attribute_boolean (info, BRASERO_IO_HAS_AUDIO)) {
 		/* audio */
 		brasero_player_create_controls_stream (player, FALSE);
 		gtk_widget_hide (player->priv->notebook);
 		gtk_range_set_value (GTK_RANGE (player->priv->progress), 0.0);
 
-		if (metadata->is_seekable)
+		if (g_file_info_get_attribute_boolean (info, BRASERO_IO_IS_SEEKABLE))
 			gtk_widget_set_sensitive (player->priv->progress, TRUE);
 		else
 			gtk_widget_set_sensitive (player->priv->progress, FALSE);
 	}
-	else if (info && info->mime_type && !strncmp ("image/", info->mime_type, 6)) {
+	else if (!strncmp ("image/", g_file_info_get_content_type (info), 6)) {
 		brasero_player_image (player);
 		return;
 	}
@@ -907,14 +928,14 @@ brasero_player_metadata_completed (BraseroVFS *vfs,
 	}
 
 	if (player->priv->end <= 0)
-		player->priv->end = metadata->len;
+		player->priv->end = g_file_info_get_attribute_uint64 (info, BRASERO_IO_LEN);
 
-	player->priv->length = metadata->len;
+	player->priv->length = g_file_info_get_attribute_uint64 (info, BRASERO_IO_LEN);
 
 	/* only reached for audio/video */
 	brasero_player_update_info_real (player,
-					 metadata->artist,
-					 metadata->title);
+					 g_file_info_get_attribute_string (info, BRASERO_IO_ARTIST),
+					 g_file_info_get_attribute_string (info, BRASERO_IO_TITLE));
 
 	player->priv->state = BACON_STATE_READY;
 	g_signal_emit (player,
@@ -925,26 +946,21 @@ brasero_player_metadata_completed (BraseroVFS *vfs,
 static void
 brasero_player_retrieve_metadata (BraseroPlayer *player)
 {
-	GList *uris;
-
-	if (!player->priv->vfs)
-		player->priv->vfs = brasero_vfs_get_default ();
+	if (!player->priv->io)
+		player->priv->io = brasero_io_get_default ();
 
 	if (!player->priv->meta_task)
-		player->priv->meta_task = brasero_vfs_register_data_type (player->priv->vfs,
-									  G_OBJECT (player),
-									  G_CALLBACK (brasero_player_metadata_completed),
-									  NULL);
+		player->priv->meta_task = brasero_io_register (G_OBJECT (player),
+							       brasero_player_metadata_completed,
+							       NULL,
+							       NULL);
 
-	uris = g_list_prepend (NULL, player->priv->uri);
-	brasero_vfs_get_metadata (player->priv->vfs,
-				  uris,
-				  GNOME_VFS_FILE_INFO_FOLLOW_LINKS,
-				  BRASERO_METADATA_FLAG_NONE,
-				  FALSE,
+	brasero_io_get_file_info (player->priv->io,
+				  player->priv->uri,
 				  player->priv->meta_task,
+				  BRASERO_IO_INFO_METADATA|
+				  BRASERO_IO_INFO_MIME,
 				  NULL);
-	g_list_free (uris);
 }
 
 const gchar *
@@ -983,7 +999,6 @@ void
 brasero_player_set_uri (BraseroPlayer *player,
 			const gchar *uri)
 {
-	gchar *song_uri;
 	gchar *name;
 
 	/* avoid reloading everything if it's the same uri */
@@ -1015,8 +1030,9 @@ brasero_player_set_uri (BraseroPlayer *player,
 	player->priv->start = 0;
 	player->priv->end = 0;
 
-	if (player->priv->vfs)
-		brasero_vfs_cancel (player->priv->vfs, player);
+	if (player->priv->io)
+		brasero_io_cancel_by_base (player->priv->io,
+					   player->priv->meta_task);
 
 	/* That stops the pipeline from playing */
 	brasero_player_bacon_set_uri (BRASERO_PLAYER_BACON (player->priv->bacon), NULL);
@@ -1029,12 +1045,15 @@ brasero_player_set_uri (BraseroPlayer *player,
 
 	if (player->priv->controls) {
 		if (player->priv->header) {
+			gchar *song_uri;
+
 			BRASERO_GET_BASENAME_FOR_DISPLAY (uri, name);
 			song_uri = g_markup_printf_escaped (_("<span weight=\"bold\">Loading information</span>\nabout <span size=\"smaller\"><i>%s</i></span>"),
 							  name);
 			g_free (name);
 
 			gtk_label_set_markup (GTK_LABEL (player->priv->header), song_uri);
+			g_free (song_uri);
 		}
 
 		/* grey out the rest of the control while it's loading */
@@ -1163,6 +1182,55 @@ brasero_player_get_pos (BraseroPlayer *self)
 	return pos;
 }
 
+static void
+brasero_player_size_allocate (GtkWidget *widget,
+			      GtkAllocation *allocation)
+{
+	BraseroPlayer *player;
+	GtkWidget *parent;
+
+	player = BRASERO_PLAYER (widget);
+	if (!player->priv->controls) {
+		GTK_WIDGET_CLASS (parent_class)->size_allocate (widget, allocation);
+		return;
+	}
+
+	if (!player->priv->pixbuf) {
+		GTK_WIDGET_CLASS (parent_class)->size_allocate (widget, allocation);
+		return;
+	}
+
+	parent = gtk_widget_get_parent (player->priv->controls);
+
+	g_object_ref (player->priv->controls);
+
+	if (allocation->width > allocation->height) {
+		if (parent != player->priv->hbox) {
+			gtk_container_remove (GTK_CONTAINER (player->priv->vbox), player->priv->controls);
+			gtk_box_pack_end (GTK_BOX (player->priv->hbox),
+					  player->priv->controls,
+					  TRUE,
+					  TRUE,
+					  0);
+		}
+	}
+	else {
+		if (parent != player->priv->vbox) {
+			gtk_container_remove (GTK_CONTAINER (player->priv->hbox), player->priv->controls);
+			gtk_box_pack_end (GTK_BOX (player->priv->vbox),
+					  player->priv->controls,
+					  TRUE,
+					  TRUE,
+					  0);
+		}
+	}
+
+	g_object_unref (player->priv->controls);
+	gtk_widget_show (player->priv->controls);
+
+	GTK_WIDGET_CLASS (parent_class)->size_allocate (widget, allocation);
+}
+
 static void
 brasero_player_destroy (GtkObject *obj)
 {
@@ -1213,13 +1281,15 @@ brasero_player_destroy (GtkObject *obj)
 	}
 
 	if (player->priv->meta_task){
-		brasero_vfs_cancel (player->priv->vfs, player);
+		brasero_io_cancel_by_base (player->priv->io,
+					   player->priv->meta_task);
+		g_free (player->priv->meta_task);
 		player->priv->meta_task = 0;
 	}
 
-	if (player->priv->vfs) {
-		g_object_unref (player->priv->vfs);
-		player->priv->vfs = NULL;
+	if (player->priv->io) {
+		g_object_unref (player->priv->io);
+		player->priv->io = NULL;
 	}
 
 	if (GTK_OBJECT_CLASS (parent_class)->destroy)
@@ -1242,10 +1312,15 @@ brasero_player_class_init (BraseroPlayerClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 	GtkObjectClass *gtk_object_class = GTK_OBJECT_CLASS (klass);
+	GtkWidgetClass *gtk_widget_class = GTK_WIDGET_CLASS (klass);
 
 	parent_class = g_type_class_peek_parent (klass);
 	object_class->finalize = brasero_player_finalize;
+
 	gtk_object_class->destroy = brasero_player_destroy;
+
+	gtk_widget_class->size_allocate = brasero_player_size_allocate;
+
 	brasero_player_signals [ERROR_SIGNAL] = 
 			g_signal_new ("error",
 				      G_TYPE_FROM_CLASS (klass),
