@@ -7,19 +7,21 @@
 ****************************************************************************/
 
 /*
- *  This program is free software; you can redistribute it and/or modify
+ *  Brasero is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Brasero is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 
@@ -35,6 +37,8 @@
 #include <glib/gi18n-lib.h>
 #include <glib-object.h>
 
+#include <gio/gio.h>
+
 #include <gtk/gtkwidget.h>
 #include <gtk/gtkvbox.h>
 #include <gtk/gtkhbox.h>
@@ -52,8 +56,6 @@
 #include <gtk/gtkscrolledwindow.h>
 #include <gtk/gtkmessagedialog.h>
 
-#include <libgnomevfs/gnome-vfs.h>
-
 #ifdef BUILD_SEARCH
 #include <beagle/beagle.h>
 #endif
@@ -63,7 +65,7 @@
 #include "brasero-playlist.h"
 #include "brasero-utils.h"
 #include "brasero-metadata.h"
-#include "brasero-vfs.h"
+#include "brasero-io.h"
 #include "eggtreemultidnd.h"
 
 #include "brasero-uri-container.h"
@@ -89,8 +91,8 @@ struct BraseroPlaylistPrivate {
 	GtkWidget *button_remove;
 	guint activity_counter;
 
-	BraseroVFS *vfs;
-	BraseroVFSDataID parse_type;
+	BraseroIO *io;
+	BraseroIOJobBase *parse_type;
 
 	gint searched:1;
 };
@@ -262,7 +264,7 @@ brasero_playlist_init (BraseroPlaylist *obj)
 	obj->priv = g_new0 (BraseroPlaylistPrivate, 1);
 	gtk_box_set_spacing (GTK_BOX (obj), BRASERO_PLAYLIST_SPACING);
 
-	obj->priv->vfs = brasero_vfs_get_default ();
+	obj->priv->io = brasero_io_get_default ();
 
 	hbox = gtk_hbox_new (FALSE, 8);
 	gtk_widget_show (hbox);
@@ -446,10 +448,15 @@ brasero_playlist_destroy (GtkObject *object)
 	/* NOTE: we must do it here since cancel could call brasero_playlist_end
 	 * itself calling decrease_activity_counter. In finalize the latter will
 	 * raise problems since the GdkWindow has been destroyed */
-	if (playlist->priv->vfs) {
-		brasero_vfs_cancel (playlist->priv->vfs, playlist);
-		g_object_unref (playlist->priv->vfs);
-		playlist->priv->vfs = NULL;
+	if (playlist->priv->io) {
+		brasero_io_cancel_by_base (playlist->priv->io,
+					   playlist->priv->parse_type);
+
+		g_free (playlist->priv->parse_type);
+		playlist->priv->parse_type = NULL;
+
+		g_object_unref (playlist->priv->io);
+		playlist->priv->io = NULL;
 	}
 
 	if (GTK_OBJECT_CLASS (parent_class)->destroy)
@@ -487,6 +494,9 @@ brasero_playlist_start_beagle_search (BraseroPlaylist *playlist)
 	playlist->priv->client = beagle_client_new (NULL);
 	if(playlist->priv->client) {
 		GError *error = NULL;
+		BeagleQueryPartOr *or_part;
+		BeagleQueryPartHuman *type;
+		BeagleQueryPartProperty *filetype;
 
 		playlist->priv->query = beagle_query_new ();
 
@@ -500,12 +510,37 @@ brasero_playlist_start_beagle_search (BraseroPlaylist *playlist)
 				  G_CALLBACK (brasero_playlist_beagle_finished_cb),
 				  playlist);
 	
-		beagle_query_add_text (playlist->priv->query, "Files");
-	
-		beagle_query_add_text (playlist->priv->query, "audio/x-scpls");
-		beagle_query_add_text (playlist->priv->query, "audio/x-ms-asx");
-		beagle_query_add_text (playlist->priv->query, "audio/x-mp3-playlist");
-		beagle_query_add_text (playlist->priv->query, "audio/x-mpegurl");
+		type = beagle_query_part_human_new ();
+		beagle_query_part_human_set_string (type, "type:File");
+		beagle_query_add_part (playlist->priv->query, BEAGLE_QUERY_PART (type));
+
+		or_part = beagle_query_part_or_new ();
+
+		filetype = beagle_query_part_property_new ();
+		beagle_query_part_property_set_property_type (filetype, BEAGLE_PROPERTY_TYPE_KEYWORD);
+		beagle_query_part_property_set_key (filetype, "beagle:MimeType");
+		beagle_query_part_property_set_value (filetype, "audio/x-ms-asx");
+		beagle_query_part_or_add_subpart (or_part, BEAGLE_QUERY_PART (filetype));
+
+		filetype = beagle_query_part_property_new ();
+		beagle_query_part_property_set_property_type (filetype, BEAGLE_PROPERTY_TYPE_KEYWORD);
+		beagle_query_part_property_set_key (filetype, "beagle:MimeType");
+		beagle_query_part_property_set_value (filetype, "audio/x-mpegurl");
+		beagle_query_part_or_add_subpart (or_part, BEAGLE_QUERY_PART (filetype));
+
+		filetype = beagle_query_part_property_new ();
+		beagle_query_part_property_set_property_type (filetype, BEAGLE_PROPERTY_TYPE_KEYWORD);
+		beagle_query_part_property_set_key (filetype, "beagle:MimeType");
+		beagle_query_part_property_set_value (filetype, "audio/x-scpls");
+		beagle_query_part_or_add_subpart (or_part, BEAGLE_QUERY_PART (filetype));
+
+		filetype = beagle_query_part_property_new ();
+		beagle_query_part_property_set_property_type (filetype, BEAGLE_PROPERTY_TYPE_KEYWORD);
+		beagle_query_part_property_set_key (filetype, "beagle:FileType");
+		beagle_query_part_property_set_value (filetype, "audio/x-mp3-playlist");
+		beagle_query_part_or_add_subpart (or_part, BEAGLE_QUERY_PART (filetype));
+
+		beagle_query_add_part (playlist->priv->query, BEAGLE_QUERY_PART (or_part));
 
 		brasero_playlist_increase_activity_counter (playlist);
 		beagle_client_send_request_async (playlist->priv->client,
@@ -617,7 +652,7 @@ brasero_playlist_add_cb (GtkButton *button, BraseroPlaylist *playlist)
 	if (!GTK_WIDGET_TOPLEVEL (toplevel))
 		return;
 
-	dialog = gtk_file_chooser_dialog_new (_("Select a playlist"),
+	dialog = gtk_file_chooser_dialog_new (_("Select Playlist"),
 					      GTK_WINDOW (toplevel),
 					      GTK_FILE_CHOOSER_ACTION_OPEN,
 					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
@@ -801,7 +836,7 @@ brasero_playlist_dialog_error (BraseroPlaylist *playlist, const gchar *uri)
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
 						  _("an unknown error occured."));
 
-	gtk_window_set_title (GTK_WINDOW (dialog), _("Playlist loading error"));
+	gtk_window_set_title (GTK_WINDOW (dialog), _("Playlist Loading Error"));
 
 	gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
@@ -809,8 +844,8 @@ brasero_playlist_dialog_error (BraseroPlaylist *playlist, const gchar *uri)
 
 static void
 brasero_playlist_parse_end (GObject *object,
-			    gpointer callback_data,
-			    gboolean cancelled)
+			    gboolean cancelled,
+			    gpointer callback_data)
 {
 	BraseroPlaylistParseData *data = callback_data;
 	BraseroPlaylist *playlist = BRASERO_PLAYLIST (object);
@@ -822,12 +857,10 @@ brasero_playlist_parse_end (GObject *object,
 }
 
 static void
-brasero_playlist_parse_result (BraseroVFS *vfs,
-			       GObject *object,
-			       GnomeVFSResult result,
+brasero_playlist_parse_result (GObject *object,
+			       GError *error,
 			       const gchar *uri,
-			       GnomeVFSFileInfo *info,
-			       BraseroMetadataInfo *metadata,
+			       GFileInfo *info,
 			       gpointer callback_data)
 {
 	gint num;
@@ -844,21 +877,27 @@ brasero_playlist_parse_result (BraseroVFS *vfs,
 	gtk_tree_model_get_iter (model, &parent, treepath);
 	gtk_tree_path_free (treepath);
 
-	if (!data->title) {
+	if (info && g_file_info_get_attribute_boolean (info, BRASERO_IO_IS_PLAYLIST)) {
+		const gchar *playlist_title = NULL;
+
 		/* The first entry returned is always the playlist as a whole:
 		 * if it was successfully parsed uri is the title if any. If not
 		 * it's simply the URI */
 
 		/* this is for the playlist as a whole */
-		if (result != GNOME_VFS_OK) {
+		if (error) {
 			if (!data->quiet)
 				brasero_playlist_dialog_error (playlist, uri);
 
 			gtk_list_store_remove (GTK_LIST_STORE (model), &parent);
+			data->title = 1;
+			return;
 		}
-		else if (uri)
+
+		playlist_title = g_file_info_get_attribute_string (info, BRASERO_IO_PLAYLIST_TITLE);
+		if (playlist_title)
 			gtk_tree_store_set (GTK_TREE_STORE (model), &parent,
-					    BRASERO_PLAYLIST_DISPLAY_COL, uri,
+					    BRASERO_PLAYLIST_DISPLAY_COL, playlist_title,
 					    -1);
 
 		data->title = 1;
@@ -866,26 +905,36 @@ brasero_playlist_parse_result (BraseroVFS *vfs,
 	}
 
     	/* See if the song can be added */
-	if (result == GNOME_VFS_OK && metadata && metadata->has_audio) {
+	if (!error && info && g_file_info_get_attribute_boolean (info, BRASERO_IO_HAS_AUDIO)) {
 		gchar *name;
+		guint64 len;
+		const gchar *title;
+		const gchar *genre;
 
 		gtk_tree_store_append (GTK_TREE_STORE (model), &row, &parent);
 
-		if (metadata->len > 0)
-			len_string = brasero_utils_get_time_string (metadata->len, TRUE, FALSE);
+		len = g_file_info_get_attribute_uint64 (info, BRASERO_IO_LEN);
+		title = g_file_info_get_attribute_string (info, BRASERO_IO_TITLE);
+		genre = g_file_info_get_attribute_string (info, BRASERO_IO_GENRE);
+
+		if (len > 0)
+			len_string = brasero_utils_get_time_string (len, TRUE, FALSE);
 		else
 			len_string = NULL;
 
 		BRASERO_GET_BASENAME_FOR_DISPLAY (uri, name);
 		gtk_tree_store_set (GTK_TREE_STORE (model), &row,
-				    BRASERO_PLAYLIST_DISPLAY_COL, metadata->title ? metadata->title : name,
+				    BRASERO_PLAYLIST_DISPLAY_COL, title ? title : name,
 				    BRASERO_PLAYLIST_LEN_COL, len_string,
-				    BRASERO_PLAYLIST_GENRE_COL, metadata->genre,
-				    BRASERO_PLAYLIST_URI_COL, metadata->uri,
-				    BRASERO_PLAYLIST_DSIZE_COL, metadata->len,
+				    BRASERO_PLAYLIST_GENRE_COL, genre,
+				    BRASERO_PLAYLIST_URI_COL, uri,
+				    BRASERO_PLAYLIST_DSIZE_COL, len,
 				    -1);
 		g_free (name);
 		g_free (len_string);
+
+		if (len)
+			total_length += len;
 	}
 
 	/* update the playlist information */
@@ -900,9 +949,6 @@ brasero_playlist_parse_result (BraseroVFS *vfs,
 			    BRASERO_PLAYLIST_DSIZE_COL, &total_length,
 			    -1);
 
-	if (metadata)
-		total_length += metadata->len;
-
   	if (total_length > 0)
 		len_string = brasero_utils_get_time_string (total_length, TRUE, FALSE);
 	else
@@ -949,45 +995,25 @@ brasero_playlist_add_uri_playlist (BraseroPlaylist *playlist,
 				   const gchar *uri,
 				   gboolean quiet)
 {
-	gboolean success;
 	BraseroPlaylistParseData *data;
 
 	data = g_new0 (BraseroPlaylistParseData, 1);
 	data->reference = brasero_playlist_insert (playlist, uri);
 
 	if (!playlist->priv->parse_type)
-		playlist->priv->parse_type = brasero_vfs_register_data_type (playlist->priv->vfs,
-									     G_OBJECT (playlist),
-									     G_CALLBACK (brasero_playlist_parse_result),
-									     brasero_playlist_parse_end);
+		playlist->priv->parse_type = brasero_io_register (G_OBJECT (playlist),
+								  brasero_playlist_parse_result,
+								  brasero_playlist_parse_end,
+								  NULL);
 
-	success = brasero_vfs_parse_playlist (playlist->priv->vfs,
-					      uri,
-					      GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS |
-					      GNOME_VFS_FILE_INFO_GET_MIME_TYPE |
-					      GNOME_VFS_FILE_INFO_FORCE_SLOW_MIME_TYPE,
-					      playlist->priv->parse_type,
-					      data);
-
-	if (!success) {
-		GtkTreePath *treepath;
-		GtkTreeModel *model;
-		GtkTreeIter iter;
-
-		/* failure: remove entry and warn user */
-		model = gtk_tree_view_get_model (GTK_TREE_VIEW (playlist->priv->tree));
-		treepath = gtk_tree_row_reference_get_path (data->reference);
-		gtk_tree_model_get_iter (model, &iter, treepath);
-		gtk_list_store_remove (GTK_LIST_STORE (model), &iter);
-		gtk_tree_path_free (treepath);
-
-		if (!quiet)
-			brasero_playlist_dialog_error (playlist, uri);
-	}
-	else {
-		data->quiet = quiet;
-		brasero_playlist_increase_activity_counter (playlist);
-	}
+	brasero_io_parse_playlist (playlist->priv->io,
+				   uri,
+				   playlist->priv->parse_type,
+				   BRASERO_IO_INFO_PERM|
+				   BRASERO_IO_INFO_MIME|
+				   BRASERO_IO_INFO_METADATA,
+				   data);
+	brasero_playlist_increase_activity_counter (playlist);
 }
 
 static void
