@@ -7,20 +7,21 @@
  * Based on gedit code (gedit/gedit-plugin-manager.c) by: 
  * 	- Paolo Maggi <paolo@gnome.org>
  *
- * This program is free software; you can redistribute it and/or modify
+ * Brasero is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * Brasero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, 
- * Boston, MA 02111-1307, USA. 
+ * along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -369,6 +370,7 @@ plugin_manager_ui_populate_lists (BraseroPluginManagerUI *pm)
 	}
 	g_object_unref (caps);
 
+	plugin = NULL;
 	if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (model), &iter))
 	{
 		GtkTreeSelection *selection;
@@ -568,6 +570,9 @@ create_tree_popup_menu (BraseroPluginManagerUI *pm)
 
 	plugin = plugin_manager_ui_get_selected_plugin (pm);
 
+	if (!plugin)
+		return NULL;
+
 	menu = gtk_menu_new ();
 
 	item = gtk_image_menu_item_new_with_mnemonic (_("_About"));
@@ -714,6 +719,8 @@ show_tree_popup_menu (GtkTreeView        *tree,
 		gtk_widget_destroy (priv->popup_menu);
 
 	priv->popup_menu = create_tree_popup_menu (pm);
+	if (!priv->popup_menu)
+		return;
 	
 	gtk_menu_attach_to_widget (GTK_MENU (priv->popup_menu),
 				   GTK_WIDGET (pm),
@@ -983,11 +990,8 @@ brasero_plugin_manager_ui_init (BraseroPluginManagerUI *pm)
 		gtk_widget_set_sensitive (priv->about_button, FALSE);
 		gtk_widget_set_sensitive (priv->configure_button, FALSE);		
 	}
-	else {
+	else
 		plugin_manager_ui_populate_lists (pm);
-		gtk_widget_set_sensitive (priv->about_button, TRUE);
-		gtk_widget_set_sensitive (priv->configure_button, TRUE);
-	}
 }
 
 static void
