@@ -7,19 +7,21 @@
  ***************************************************************************/
 
 /*
- *  This program is free software; you can redistribute it and/or modify
+ *  Brasero is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Brasero is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 #include <string.h>
@@ -43,10 +45,6 @@
 #include <gtk/gtkuimanager.h>
 #include <gtk/gtkscrolledwindow.h>
 
-#include <libgnomevfs/gnome-vfs.h>
-#include <libgnomevfs/gnome-vfs-file-info.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
-
 #include "brasero-utils.h"
 #include "brasero-project.h"
 #include "brasero-layout.h"
@@ -57,7 +55,7 @@
 #include "brasero-burn-dialog.h"
 #include "brasero-project-type-chooser.h"
 #include "brasero-disc-copy-dialog.h"
-#include "brasero-vfs.h"
+#include "brasero-io.h"
 #include "burn-caps.h"
 
 #ifdef BUILD_SEARCH
@@ -81,6 +79,8 @@ brasero_project_manager_type_changed_cb (BraseroProjectTypeChooser *chooser,
 					 BraseroProjectType type,
 					 BraseroProjectManager *manager);
 
+static void
+brasero_project_manager_new_cover_cb (GtkAction *action, BraseroProjectManager *manager);
 static void
 brasero_project_manager_new_empty_prj_cb (GtkAction *action, BraseroProjectManager *manager);
 static void
@@ -88,6 +88,8 @@ brasero_project_manager_new_audio_prj_cb (GtkAction *action, BraseroProjectManag
 static void
 brasero_project_manager_new_data_prj_cb (GtkAction *action, BraseroProjectManager *manager);
 static void
+brasero_project_manager_new_video_prj_cb (GtkAction *action, BraseroProjectManager *manager);
+static void
 brasero_project_manager_new_copy_prj_cb (GtkAction *action, BraseroProjectManager *manager);
 static void
 brasero_project_manager_new_iso_prj_cb (GtkAction *action, BraseroProjectManager *manager);
@@ -107,7 +109,9 @@ brasero_project_manager_selected_uris_changed (BraseroURIContainer *container,
 
 /* menus */
 static GtkActionEntry entries [] = {
-	{"New", GTK_STOCK_NEW, N_("_New Project"), NULL,
+	{"Cover", NULL, N_("_Cover Editor"), NULL,
+	 N_("Design and print covers for CDs"), G_CALLBACK (brasero_project_manager_new_cover_cb)},
+	 {"New", GTK_STOCK_NEW, N_("_New Project"), NULL,
 	 N_("Create a new project"), NULL },
 	{"NewChoose", GTK_STOCK_NEW, N_("_Empty Project"), NULL,
 	 N_("Let you choose your new project"), G_CALLBACK (brasero_project_manager_new_empty_prj_cb)},
@@ -115,6 +119,8 @@ static GtkActionEntry entries [] = {
 	 N_("Create a new audio project"), G_CALLBACK (brasero_project_manager_new_audio_prj_cb)},
 	{"NewData", "media-optical-data-new", N_("New _Data Project"), NULL,
 	 N_("Create a new data project"), G_CALLBACK (brasero_project_manager_new_data_prj_cb)},
+	{"NewVideo", "media-optical-video-new", N_("New _Video Project"), NULL,
+	 N_("Create a new video project"), G_CALLBACK (brasero_project_manager_new_video_prj_cb)},
 	{"NewCopy", "media-optical-copy", N_("Copy _Disc..."), NULL,
 	 N_("Copy a disc"), G_CALLBACK (brasero_project_manager_new_copy_prj_cb)},
 	{"NewIso", "iso-image-burn", N_("_Burn Image..."), NULL,
@@ -129,13 +135,13 @@ static const char *description = {
 	    "<menubar name='menubar' >"
 		"<menu action='ProjectMenu'>"
 			"<placeholder name='ProjectPlaceholder'>"
-				"<placeholder name='ViewPlaceholder'/>"
 				"<menu action='New'>"
 					"<menuitem action='NewAudio'/>"
 					"<menuitem action='NewData'/>"
+					"<menuitem action='NewVideo'/>"
 					"<menuitem action='NewCopy'/>"	
 					"<menuitem action='NewIso'/>"	
-			"</menu>"
+				"</menu>"
 			"</placeholder>"
 
 			"<placeholder name='ProjectPlaceholder'>"
@@ -144,15 +150,26 @@ static const char *description = {
 			    "<menuitem action='RecentProjects'/>"
 			    "<separator/>"
 			"</placeholder>"
+
+		"</menu>"
+
+		"<menu action='ToolMenu'>"
+
+			"<placeholder name='DiscPlaceholder'>"
+			    "<separator/>"
+			    "<menuitem action='Cover'/>"
+			    "<separator/>"
+			"</placeholder>"
+
 		"</menu>"
 	    "</menubar>"
 	"</ui>"
 };
 
 struct BraseroProjectManagerPrivate {
-	BraseroVFS *vfs;
+	BraseroIO *io;
 	BraseroProjectType type;
-	BraseroVFSDataID size_preview;
+	BraseroIOJobBase *size_preview;
 
 	GtkWidget *project;
 	GtkWidget *layout;
@@ -199,27 +216,40 @@ brasero_project_manager_get_type ()
 }
 
 static void
-brasero_project_manager_size_preview (BraseroVFS *vfs,
-				      GObject *object,
-				      gint files_num,
-				      gint invalid_num,
-				      gint64 files_size,
-				      gpointer user_data)
+brasero_project_manager_new_cover_cb (GtkAction *action,
+				      BraseroProjectManager *manager)
 {
-	BraseroProjectManager *manager = BRASERO_PROJECT_MANAGER (object);
-	gint valid_num = files_num - invalid_num;
-    	gchar *status_string = NULL;
+	GtkWidget *toplevel;
+	GtkWidget *edit;
+
+	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (manager));
+	edit = brasero_jacket_edit_dialog_new (toplevel, NULL);
+
+	if (manager->priv->type == BRASERO_PROJECT_TYPE_AUDIO)
+		brasero_project_set_cover_specifics (BRASERO_PROJECT (manager->priv->project),
+						     BRASERO_JACKET_EDIT (edit));
+}
+
+static void
+brasero_project_manager_set_statusbar (BraseroProjectManager *manager,
+				       guint64 files_size,
+				       gint invalid_num,
+				       gint files_num)
+{
+	gchar *status_string = NULL;
+	gint valid_num;
 
 	gtk_statusbar_pop (GTK_STATUSBAR (manager->priv->status),
 			   manager->priv->status_ctx);
 
+	valid_num = files_num - invalid_num;
 	if (!invalid_num && valid_num) {
 		gchar *size_string;
 
 		if (manager->priv->type == BRASERO_PROJECT_TYPE_AUDIO)
 			size_string = brasero_utils_get_time_string (files_size, TRUE, FALSE);
 		else if (manager->priv->type == BRASERO_PROJECT_TYPE_DATA)
-			size_string = gnome_vfs_format_file_size_for_display (files_size);
+			size_string = g_format_size_for_display (files_size);
 		else
 			return;
 
@@ -238,7 +268,7 @@ brasero_project_manager_size_preview (BraseroVFS *vfs,
 							 size_string);
 		}
 		else if (manager->priv->type == BRASERO_PROJECT_TYPE_DATA) {
-			size_string = gnome_vfs_format_file_size_for_display (files_size);
+			size_string = g_format_size_for_display (files_size);
 			status_string = g_strdup_printf (ngettext ("%d file can be added (%s)", "%d selected files can be added (%s)", valid_num),
 							 valid_num,
 							 size_string);
@@ -269,20 +299,60 @@ brasero_project_manager_size_preview (BraseroVFS *vfs,
 	g_free (status_string);
 }
 
+static void
+brasero_project_manager_size_preview (GObject *object,
+				      GError *error,
+				      const gchar *uri,
+				      GFileInfo *info,
+				      gpointer user_data)
+{
+	BraseroProjectManager *manager = BRASERO_PROJECT_MANAGER (object);
+	guint64 files_size;
+	gint invalid_num;
+	gint files_num;
+
+	invalid_num = g_file_info_get_attribute_uint32 (info, BRASERO_IO_COUNT_INVALID);
+	files_size = g_file_info_get_attribute_uint64 (info, BRASERO_IO_COUNT_SIZE);
+	files_num = g_file_info_get_attribute_uint32 (info, BRASERO_IO_COUNT_NUM);
+	brasero_project_manager_set_statusbar (manager,
+					       files_size,
+					       invalid_num,
+					       files_num);
+}
+
+static void
+brasero_project_manager_size_preview_progress (GObject *object,
+					       BraseroIOJobProgress *progress,
+					       gpointer user_data)
+{
+	BraseroProjectManager *manager = BRASERO_PROJECT_MANAGER (object);
+	guint64 files_size;
+	gint files_num;
+
+	files_size = brasero_io_job_progress_get_total (progress);
+	files_num = brasero_io_job_progress_get_file_processed (progress);
+	brasero_project_manager_set_statusbar (manager,
+					       files_size,
+					       0,
+					       files_num);
+}
+
 void
 brasero_project_manager_selected_uris_changed (BraseroURIContainer *container,
 					       BraseroProjectManager *manager)
 {
+	BraseroIOFlags flags;
     	gchar **uris, **iter;
-    	GList *list = NULL;
+    	GSList *list = NULL;
 
 	/* if we are in the middle of an unfinished size seek then
 	 * cancel it and re-initialize */
-	if (manager->priv->vfs)
-		brasero_vfs_cancel (manager->priv->vfs, manager);
+	if (manager->priv->io)
+		brasero_io_cancel_by_base (manager->priv->io,
+					   manager->priv->size_preview);
 
 	uris = brasero_uri_container_get_selected_uris (container);
-    	if (!uris) {
+	if (!uris) {
 		gtk_statusbar_pop (GTK_STATUSBAR (manager->priv->status),
 				   manager->priv->status_ctx);
 		gtk_statusbar_push (GTK_STATUSBAR (manager->priv->status),
@@ -291,24 +361,29 @@ brasero_project_manager_selected_uris_changed (BraseroURIContainer *container,
 		return;
 	}
 
-    	for (iter = uris; iter && *iter; iter ++)
-		list = g_list_prepend (list, *iter);
-
-	if (!manager->priv->vfs)
-		manager->priv->vfs = brasero_vfs_get_default ();
+	if (!manager->priv->io)
+		manager->priv->io = brasero_io_get_default ();
 
 	if (!manager->priv->size_preview)
-		manager->priv->size_preview = brasero_vfs_register_data_type (manager->priv->vfs,
-									      G_OBJECT (manager),
-									      G_CALLBACK (brasero_project_manager_size_preview),
-									      NULL);
-	brasero_vfs_get_count (manager->priv->vfs,
-			       list,
-			      (manager->priv->type == BRASERO_PROJECT_TYPE_AUDIO),
-			       manager->priv->size_preview,
-			       NULL);
+		manager->priv->size_preview = brasero_io_register (G_OBJECT (manager),
+								   brasero_project_manager_size_preview,
+								   NULL,
+								   brasero_project_manager_size_preview_progress);
+    
+	for (iter = uris; iter && *iter; iter ++)
+		list = g_slist_prepend (list, *iter);
+
+	flags = BRASERO_IO_INFO_RECURSIVE|BRASERO_IO_INFO_IDLE;
+	if (manager->priv->type == BRASERO_PROJECT_TYPE_AUDIO)
+		flags |= BRASERO_IO_INFO_METADATA;
+
+	brasero_io_get_file_count (manager->priv->io,
+				   list,
+				   manager->priv->size_preview,
+				   flags,
+				   NULL);
 			       
-	g_list_free (list);
+	g_slist_free (list);
 	g_strfreev (uris);
 }
 void
@@ -352,10 +427,11 @@ brasero_project_manager_burn (BraseroProjectManager *manager,
 	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
 
 	gtk_widget_hide (toplevel);
-	gtk_widget_show_all (dialog);
+	gtk_widget_show (dialog);
 
-	destroy = brasero_burn_dialog_run (BRASERO_BURN_DIALOG (dialog),
-					   session);
+	brasero_burn_dialog_run (BRASERO_BURN_DIALOG (dialog),
+				 session,
+				 &destroy);
 
 	gtk_widget_destroy (dialog);
 
@@ -421,7 +497,7 @@ brasero_project_manager_burn_disc (BraseroProjectManager *manager)
 	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (toplevel));
 	gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
 	gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER_ON_PARENT);
-	gtk_widget_show_all (dialog);
+	gtk_widget_show (dialog);
 
 	result = gtk_dialog_run (GTK_DIALOG (dialog));
 	if (result != GTK_RESPONSE_OK) {
@@ -447,7 +523,8 @@ brasero_project_manager_switch (BraseroProjectManager *manager,
 	GtkAction *action;
 
 	if ((manager->priv->type == BRASERO_PROJECT_TYPE_AUDIO
-	||   manager->priv->type == BRASERO_PROJECT_TYPE_DATA)
+	||   manager->priv->type == BRASERO_PROJECT_TYPE_DATA
+	||   manager->priv->type == BRASERO_PROJECT_TYPE_VIDEO)
 	&&  !brasero_project_confirm_switch (BRASERO_PROJECT (manager->priv->project)))
 		return;
 
@@ -481,7 +558,7 @@ brasero_project_manager_switch (BraseroProjectManager *manager,
 		}
 
 		if (toplevel)
-			gtk_window_set_title (GTK_WINDOW (toplevel), _("Brasero - New audio disc project"));
+			gtk_window_set_title (GTK_WINDOW (toplevel), _("Brasero - New Audio Disc Project"));
 	}
 	else if (type == BRASERO_PROJECT_TYPE_DATA) {
 		brasero_layout_load (BRASERO_LAYOUT (manager->priv->layout), BRASERO_LAYOUT_DATA);
@@ -495,7 +572,21 @@ brasero_project_manager_switch (BraseroProjectManager *manager,
 		}
 
 		if (toplevel)
-			gtk_window_set_title (GTK_WINDOW (toplevel), _("Brasero - New data disc project"));
+			gtk_window_set_title (GTK_WINDOW (toplevel), _("Brasero - New Data Disc Project"));
+	}
+	else if (type == BRASERO_PROJECT_TYPE_VIDEO) {
+		brasero_layout_load (BRASERO_LAYOUT (manager->priv->layout), BRASERO_LAYOUT_VIDEO);
+
+		gtk_notebook_set_current_page (GTK_NOTEBOOK (manager), 1);
+		gtk_action_set_sensitive (action, TRUE);
+
+		if (reset) {
+			/* tell the BraseroProject object that we want a data selection */
+			brasero_project_set_video (BRASERO_PROJECT (manager->priv->project), uris);
+		}
+
+		if (toplevel)
+			gtk_window_set_title (GTK_WINDOW (toplevel), _("Brasero - New Video disc project"));
 	}
 	else if (type == BRASERO_PROJECT_TYPE_ISO) {
 		brasero_layout_load (BRASERO_LAYOUT (manager->priv->layout), BRASERO_LAYOUT_NONE);
@@ -505,7 +596,7 @@ brasero_project_manager_switch (BraseroProjectManager *manager,
 		gtk_action_set_sensitive (action, FALSE);
 
 		if (toplevel)
-			gtk_window_set_title (GTK_WINDOW (toplevel), _("Brasero - New image file"));
+			gtk_window_set_title (GTK_WINDOW (toplevel), _("Brasero - New Image File"));
 		brasero_project_manager_burn_iso_dialog (manager, uri);
 	}
 	else if (type == BRASERO_PROJECT_TYPE_COPY) {
@@ -516,7 +607,7 @@ brasero_project_manager_switch (BraseroProjectManager *manager,
 		gtk_action_set_sensitive (action, FALSE);
 
 		if (toplevel)
-			gtk_window_set_title (GTK_WINDOW (toplevel), _("Brasero - Copy a disc"));
+			gtk_window_set_title (GTK_WINDOW (toplevel), _("Brasero - Disc Copy"));
 
 		brasero_project_manager_burn_disc (manager);
 	}
@@ -548,6 +639,12 @@ brasero_project_manager_new_data_prj_cb (GtkAction *action, BraseroProjectManage
 	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_DATA, NULL, NULL, TRUE);
 }
 
+static void
+brasero_project_manager_new_video_prj_cb (GtkAction *action, BraseroProjectManager *manager)
+{
+	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_VIDEO, NULL, NULL, TRUE);
+}
+
 static void
 brasero_project_manager_new_copy_prj_cb (GtkAction *action, BraseroProjectManager *manager)
 {
@@ -560,27 +657,6 @@ brasero_project_manager_new_iso_prj_cb (GtkAction *action, BraseroProjectManager
 	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_ISO, NULL, NULL, TRUE);
 }
 
-static void
-brasero_project_manager_open_cb (GtkAction *action, BraseroProjectManager *manager)
-{
-	BraseroProjectType type;
-
-	type = brasero_project_open_project (BRASERO_PROJECT (manager->priv->project), NULL);
-	if (type == BRASERO_PROJECT_TYPE_INVALID)
-		return;
-
-	manager->priv->type = type;
-
-	if (type == BRASERO_PROJECT_TYPE_DATA)
-		brasero_layout_load (BRASERO_LAYOUT (manager->priv->layout), BRASERO_LAYOUT_DATA);
-	else
-		brasero_layout_load (BRASERO_LAYOUT (manager->priv->layout), BRASERO_LAYOUT_AUDIO);
-	gtk_notebook_set_current_page (GTK_NOTEBOOK (manager), 1);
-
-	action = gtk_action_group_get_action (manager->priv->action_group, "NewChoose");
-	gtk_action_set_sensitive (action, TRUE);
-}
-
 void
 brasero_project_manager_audio (BraseroProjectManager *manager, GSList *uris)
 {
@@ -601,6 +677,16 @@ brasero_project_manager_data (BraseroProjectManager *manager, GSList *uris)
 					TRUE);
 }
 
+void
+brasero_project_manager_video (BraseroProjectManager *manager, GSList *uris)
+{
+	brasero_project_manager_switch (manager,
+					BRASERO_PROJECT_TYPE_VIDEO,
+					uris,
+					NULL,
+					TRUE);
+}
+
 void
 brasero_project_manager_copy (BraseroProjectManager *manager)
 {
@@ -613,16 +699,9 @@ brasero_project_manager_iso (BraseroProjectManager *manager, const gchar *uri)
 	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_ISO, NULL, uri, TRUE);
 }
 
-static void
-brasero_project_manager_uri_clicked_cb (BraseroProjectTypeChooser *chooser,
-					const gchar *uri,
-					BraseroProjectManager *manager)
-{
-	brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_ISO, NULL, uri, TRUE);
-}
-
-void
-brasero_project_manager_open (BraseroProjectManager *manager, const gchar *uri)
+BraseroProjectType
+brasero_project_manager_open_project (BraseroProjectManager *manager,
+				      const gchar *uri)
 {
 	BraseroProjectType type;
 	GtkAction *action;
@@ -635,7 +714,7 @@ brasero_project_manager_open (BraseroProjectManager *manager, const gchar *uri)
 
     	if (type == BRASERO_PROJECT_TYPE_INVALID) {
 		brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_INVALID, NULL, NULL, TRUE);
-		return;
+		return BRASERO_PROJECT_TYPE_INVALID;
 	}
 
 	if (type == BRASERO_PROJECT_TYPE_DATA)
@@ -645,14 +724,155 @@ brasero_project_manager_open (BraseroProjectManager *manager, const gchar *uri)
 
 	action = gtk_action_group_get_action (manager->priv->action_group, "NewChoose");
 	gtk_action_set_sensitive (action, TRUE);
+
+	return type;
+}
+
+#ifdef BUILD_PLAYLIST
+
+BraseroProjectType
+brasero_project_manager_open_playlist (BraseroProjectManager *manager,
+				       const gchar *uri)
+{
+	BraseroProjectType type;
+	GtkAction *action;
+
+    	gtk_widget_show (manager->priv->layout);
+	gtk_notebook_set_current_page (GTK_NOTEBOOK (manager), 1);
+	type = brasero_project_open_playlist (BRASERO_PROJECT (manager->priv->project), uri);
+	manager->priv->type = type;
+
+    	if (type == BRASERO_PROJECT_TYPE_INVALID) {
+		brasero_project_manager_switch (manager, BRASERO_PROJECT_TYPE_INVALID, NULL, NULL, TRUE);
+		return BRASERO_PROJECT_TYPE_INVALID;
+	}
+
+	brasero_layout_load (BRASERO_LAYOUT (manager->priv->layout), BRASERO_LAYOUT_AUDIO);
+	action = gtk_action_group_get_action (manager->priv->action_group, "NewChoose");
+	gtk_action_set_sensitive (action, TRUE);
+
+	return BRASERO_PROJECT_TYPE_AUDIO;
+}
+
+#endif
+
+BraseroProjectType
+brasero_project_manager_open_by_mime (BraseroProjectManager *manager,
+				      const gchar *uri,
+				      const gchar *mime)
+{
+	if (!strcmp (mime, "application/x-brasero"))
+		return brasero_project_manager_open_project (manager, uri);
+
+#ifdef BUILD_PLAYLIST
+
+	else if (!strcmp (mime, "audio/x-scpls")
+	     ||  !strcmp (mime, "audio/x-ms-asx")
+	     ||  !strcmp (mime, "audio/x-mp3-playlist")
+	     ||  !strcmp (mime, "audio/x-mpegurl"))
+		return brasero_project_manager_open_playlist (manager, uri);
+#endif
+
+	else if (!strcmp (mime, "application/x-cd-image")
+	     ||  !strcmp (mime, "application/x-cdrdao-toc")
+	     ||  !strcmp (mime, "application/x-toc")
+	     ||  !strcmp (mime, "application/x-cue")) {
+		brasero_project_manager_iso (manager, uri);
+		return BRASERO_PROJECT_TYPE_ISO;
+	}
+
+	return BRASERO_PROJECT_TYPE_INVALID;
+}
+
+BraseroProjectType
+brasero_project_manager_open_uri (BraseroProjectManager *manager,
+				  const gchar *uri_arg)
+{
+  	GtkWidget *dialog;
+	GtkWidget *window;
+	gchar *uri;
+	GFile *file;
+	GFileInfo *info;
+	const gchar *mime;
+	BraseroProjectType type;
+
+	/* FIXME: make that asynchronous */
+	file = g_file_new_for_commandline_arg (uri_arg);
+	info = g_file_query_info (file,
+				  G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
+				  G_FILE_QUERY_INFO_NONE,
+				  NULL,
+				  NULL);
+
+	uri = g_file_get_uri (file);
+	mime = g_file_info_get_content_type (info);
+
+	if (g_file_query_exists (file, NULL)) {
+	  	type = brasero_project_manager_open_by_mime (manager, uri, mime);
+        } 
+	else {
+		window = gtk_widget_get_toplevel (GTK_WIDGET (manager));
+	  	dialog = gtk_message_dialog_new (GTK_WINDOW (window),
+					   	 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
+					   	 GTK_MESSAGE_ERROR,
+					   	 GTK_BUTTONS_CLOSE,
+					   	 "Error loading project");
+	  	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+							  _("The project '%s' does not exist."),
+							  uri);
+	  	gtk_dialog_run (GTK_DIALOG (dialog));
+	  	gtk_widget_destroy (dialog);
+
+		type = BRASERO_PROJECT_TYPE_INVALID;
+	}
+
+	g_free (uri);
+	g_object_unref (file);
+	g_object_unref (info);
+
+	return type;
 }
 
 static void
-brasero_project_manager_project_clicked_cb (BraseroProjectTypeChooser *chooser,
-					    const gchar *uri,
-					    BraseroProjectManager *manager)
+brasero_project_manager_open_cb (GtkAction *action, BraseroProjectManager *manager)
 {
-	brasero_project_manager_open (manager, uri);
+	gchar *uri;
+	gint answer;
+	GtkWidget *chooser;
+	GtkWidget *toplevel;
+	BraseroProjectType type;
+
+	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (manager));
+	chooser = gtk_file_chooser_dialog_new (_("Open Project"),
+					      GTK_WINDOW (toplevel),
+					      GTK_FILE_CHOOSER_ACTION_OPEN,
+					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					      GTK_STOCK_OPEN, GTK_RESPONSE_OK,
+					      NULL);
+	gtk_file_chooser_set_local_only (GTK_FILE_CHOOSER (chooser), TRUE);
+	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser),
+					     g_get_home_dir ());
+	
+	gtk_widget_show (chooser);
+	answer = gtk_dialog_run (GTK_DIALOG (chooser));
+	if (answer != GTK_RESPONSE_OK) {
+		gtk_widget_destroy (chooser);
+		return;
+	}
+
+	uri = gtk_file_chooser_get_uri (GTK_FILE_CHOOSER (chooser));
+	gtk_widget_destroy (chooser);
+
+	type = brasero_project_manager_open_uri (manager, uri);
+	g_free (uri);
+}
+
+static void
+brasero_project_manager_recent_clicked_cb (BraseroProjectTypeChooser *chooser,
+					   const gchar *uri,
+					   BraseroProjectManager *manager)
+{
+	brasero_project_manager_open_uri (manager, uri);
 }
 
 void
@@ -669,9 +889,8 @@ brasero_project_manager_load_session (BraseroProjectManager *manager,
 		gchar *uri;
 		BraseroProjectType type;
 
-		uri = gnome_vfs_make_uri_from_input (path);
-    		type = brasero_project_load_session (BRASERO_PROJECT (manager->priv->project),
-						     uri);
+		uri = g_filename_to_uri (path, NULL, NULL);
+    		type = brasero_project_load_session (BRASERO_PROJECT (manager->priv->project), uri);
 		g_free (uri);
 
 		brasero_project_manager_switch (manager, type, NULL, NULL, FALSE);
@@ -693,7 +912,7 @@ brasero_project_manager_save_session (BraseroProjectManager *manager,
 		/* if we want to save the current open project, this need a
 		 * modification in BraseroProject to bypass ask_status in case
 	 	 * DataDisc has not finished exploration */
-		uri = gnome_vfs_make_uri_from_input (path);
+		uri = g_filename_to_uri (path, NULL, NULL);
     		result = brasero_project_save_session (BRASERO_PROJECT (manager->priv->project),
 						       uri,
 						       cancellable);
@@ -739,12 +958,8 @@ brasero_project_manager_init (BraseroProjectManager *obj)
 			  G_CALLBACK (brasero_project_manager_type_changed_cb),
 			  obj);
 	g_signal_connect (type,
-			  "uri-clicked",
-			  G_CALLBACK (brasero_project_manager_uri_clicked_cb),
-			  obj);
-	g_signal_connect (type,
-			  "project-clicked",
-			  G_CALLBACK (brasero_project_manager_project_clicked_cb),
+			  "recent-clicked",
+			  G_CALLBACK (brasero_project_manager_recent_clicked_cb),
 			  obj);
 	gtk_notebook_prepend_page (GTK_NOTEBOOK (obj), type, NULL);
 
@@ -775,9 +990,9 @@ brasero_project_manager_init (BraseroProjectManager *obj)
 				   chooser,
 				   "Chooser",
 				   _("Browse the file system"),
-				   _("Display file browser"),
+				   _("Display File Browser"),
 				   GTK_STOCK_DIRECTORY,
-				   BRASERO_LAYOUT_AUDIO|BRASERO_LAYOUT_DATA);
+				   BRASERO_LAYOUT_AUDIO|BRASERO_LAYOUT_DATA|BRASERO_LAYOUT_VIDEO);
 
 #ifdef BUILD_PREVIEW
 	brasero_preview_add_source (BRASERO_PREVIEW (preview),
@@ -786,7 +1001,7 @@ brasero_project_manager_init (BraseroProjectManager *obj)
 
 	brasero_layout_add_project (BRASERO_LAYOUT (obj->priv->layout),
 				    obj->priv->project);
-	gtk_widget_show_all (obj->priv->project);
+	gtk_widget_show (obj->priv->project);
 
 #ifdef BUILD_SEARCH
 	GtkWidget *search;
@@ -799,9 +1014,9 @@ brasero_project_manager_init (BraseroProjectManager *obj)
 				   search,
 				   "Search",
 				   _("Search files using keywords"),
-				   _("Display search"),
+				   _("Display Search"),
 				   GTK_STOCK_FIND,
-				   BRASERO_LAYOUT_AUDIO|BRASERO_LAYOUT_DATA);
+				   BRASERO_LAYOUT_AUDIO|BRASERO_LAYOUT_DATA|BRASERO_LAYOUT_VIDEO);
 
 #ifdef BUILD_PREVIEW
 	brasero_preview_add_source (BRASERO_PREVIEW (preview),
@@ -820,7 +1035,7 @@ brasero_project_manager_init (BraseroProjectManager *obj)
 				   playlist,
 				   "Playlist",
 				   _("Display playlists and their contents"),
-				   _("Display playlists"),
+				   _("Display Playlists"),
 				   "audio-x-generic", 
 				   BRASERO_LAYOUT_AUDIO);
 
@@ -845,10 +1060,14 @@ brasero_project_manager_finalize (GObject *object)
 
 	cobj = BRASERO_PROJECT_MANAGER (object);
 
-	if (cobj->priv->vfs) {
-		brasero_vfs_cancel (cobj->priv->vfs, object);
-		g_object_unref (cobj->priv->vfs);
-		cobj->priv->vfs = NULL;
+	if (cobj->priv->io) {
+		brasero_io_cancel_by_base (cobj->priv->io, cobj->priv->size_preview);
+
+		g_free (cobj->priv->size_preview);
+		cobj->priv->size_preview = NULL;
+
+		g_object_unref (cobj->priv->io);
+		cobj->priv->io = NULL;
 	}
 
 	g_free (cobj->priv);
