@@ -1,17 +1,19 @@
 /*
- * This program is free software; you can redistribute it and/or modify
+ * Brasero is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  * 
- * This program is distributed in the hope that it will be useful,
+ * Brasero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU Library General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
  
 /***************************************************************************
@@ -22,13 +24,16 @@
  *  <brasero-app@wanadoo.fr>
  ****************************************************************************/
 
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
 #include <string.h>
 
 #include <glib.h>
 #include <glib/gstdio.h>
 
-#include <libgnome/gnome-util.h>
-#include <libgnomeui/gnome-client.h>
+#include <gtk/gtk.h>
 
 #include <libxml/xmlerror.h>
 #include <libxml/xmlwriter.h>
@@ -39,8 +44,6 @@
 #include "brasero-session.h"
 #include "brasero-project-manager.h"
 
-static GnomeClient *client = NULL;
-
 #define SESSION_VERSION "0.1"
 
 static gchar *
@@ -299,6 +302,16 @@ error:
 	return FALSE;
 }
 
+#ifdef BUILD_GNOME2
+
+/**
+ * This code is for session management
+ */
+
+#include <libgnomeui/gnome-client.h>
+
+static GnomeClient *client = NULL;
+
 static void
 brasero_session_die_cb (GnomeClient *client_loc,
 			BraseroApp *app)
@@ -365,3 +378,5 @@ brasero_session_disconnect (BraseroApp *app)
 
 	client = NULL;
 }
+
+#endif
