@@ -1,14 +1,12 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /*
  * brasero
- * Copyright (C) Philippe Rouquier 2007 <bonfire-app@wanadoo.fr>
+ * Copyright (C) Philippe Rouquier 2007-2008 <bonfire-app@wanadoo.fr>
  * 
- * brasero is free software.
- * 
- * You may redistribute it and/or modify it under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option)
- * any later version.
+ *  Brasero is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  * 
  * brasero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -171,11 +169,19 @@ brasero_split_dialog_size_error (BraseroSplitDialog *self)
 					  GTK_DIALOG_DESTROY_WITH_PARENT|
 					  GTK_DIALOG_MODAL,
 					  GTK_MESSAGE_QUESTION,
-					  GTK_BUTTONS_YES_NO,
-					  _("The size of the new track is shorter than 6 seconds and will be padded."));
-	gtk_window_set_title (GTK_WINDOW (message), _("size error"));
+					  GTK_BUTTONS_NONE,
+					  _("Do you really want to split the track?"));
+
+	gtk_window_set_title (GTK_WINDOW (message), _("Size Error"));
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message),
-						  _("Do you want to split it nevertheless?"));
+						  _("If you split the track, the size of the new track will be shorter than 6 seconds and will be padded."));
+
+	gtk_dialog_add_button (GTK_DIALOG (message),
+			       _("_Don't split"),
+			       GTK_RESPONSE_CANCEL);
+	gtk_dialog_add_button (GTK_DIALOG (message),
+			       _("_Split"),
+			       GTK_RESPONSE_YES);
 
 	answer = gtk_dialog_run (GTK_DIALOG (message));
 	gtk_widget_destroy (message);
@@ -192,7 +198,9 @@ brasero_split_dialog_cut (BraseroSplitDialog *self,
 			  gboolean warn)
 {
 	BraseroSplitDialogPrivate *priv;
+	BraseroAudioSlice slice = {0,0};
 	GtkTreeModel *model;
+	GtkTreeIter child;
 	GtkTreeIter iter;
 	gchar *length_str;
 	gchar *start_str;
@@ -266,8 +274,8 @@ brasero_split_dialog_cut (BraseroSplitDialog *self,
 		return TRUE;
 	}
 
+	/* Try to find an already created slice encompassing the position */
 	do {
-		GtkTreeIter child;
 		gint64 start;
 		gint64 end;
 
@@ -291,8 +299,51 @@ brasero_split_dialog_cut (BraseroSplitDialog *self,
 		&& !brasero_split_dialog_size_error (self))
 			return FALSE;
 
+		/* Found one */
+		slice.start = start;
+		slice.end = end;
+		break;
+
+	} while (gtk_tree_model_iter_next (model, &iter));
+
+	/* see if we found a slice, if not create a new one starting at pos
+	 * until the end of the song */
+
+	if (slice.start == 0 && slice.end == 0) {
+		slice.start = pos;
+
+		/* check if we need to stop this slice at the end of the song
+		 * or at the start of the next slice. */
+		if (gtk_tree_model_get_iter_first (model, &iter)) {
+			do {
+				gint64 start;
+				gint64 end;
+
+				gtk_tree_model_get (model, &iter,
+						    START_COL, &start,
+						    END_COL, &end,
+						    -1);
+
+				if (pos >= start)
+					continue;
+
+				/* Found one */
+				slice.end = start - 1;
+			} while (gtk_tree_model_iter_next (model, &iter));
+		}
+
+		if (!slice.end)
+			slice.end = priv->end;
+
+		/* check the size of the new slice */
+		if (warn
+		&& (slice.end - slice.start) < BRASERO_MIN_AUDIO_TRACK_LENGTH
+		&& !brasero_split_dialog_size_error (self))
+			return FALSE;
+	}
+	else {
 		/* we are in the middle of an existing slice */
-		length_str = brasero_utils_get_time_string (pos - start, TRUE, FALSE);
+		length_str = brasero_utils_get_time_string (pos - slice.start, TRUE, FALSE);
 		end_str = brasero_utils_get_time_string (pos, TRUE, FALSE);
 
 		gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (model),
@@ -300,36 +351,37 @@ brasero_split_dialog_cut (BraseroSplitDialog *self,
 								&iter);
 
 		gtk_list_store_set (priv->model, &child,
-				    END_COL, (gint64)  pos,
-				    LENGTH_COL, (gint64) (pos - start),
+				    END_COL, (gint64) pos,
+				    LENGTH_COL, (gint64) (pos - slice.start),
 				    END_STR_COL, end_str,
 				    LENGTH_STR_COL, length_str,
 				    -1);
 		g_free (length_str);
 		g_free (end_str);
 
-		gtk_list_store_append (priv->model, &child);
-
+		/* move the position by one */
 		pos ++;
-		length_str = brasero_utils_get_time_string (end - pos, TRUE, FALSE);
-		start_str = brasero_utils_get_time_string (pos, TRUE, FALSE);
-		end_str = brasero_utils_get_time_string (end, TRUE, FALSE);
+	}
 
-		gtk_list_store_set (priv->model, &child,
-				    START_COL, pos,
-				    END_COL, end,
-				    LENGTH_COL, (gint64) (end - pos),
-				    START_STR_COL, start_str,
-				    END_STR_COL, end_str,
-				    LENGTH_STR_COL, length_str,
-				    -1);
+	/* create a new one */
+	gtk_list_store_append (priv->model, &child);
 
-		g_free (length_str);
-		g_free (start_str);
-		g_free (end_str);
-		break;
+	length_str = brasero_utils_get_time_string (slice.end - pos, TRUE, FALSE);
+	start_str = brasero_utils_get_time_string (pos, TRUE, FALSE);
+	end_str = brasero_utils_get_time_string (slice.end, TRUE, FALSE);
 
-	} while (gtk_tree_model_iter_next (model, &iter));
+	gtk_list_store_set (priv->model, &child,
+			    START_COL, pos,
+			    END_COL, slice.end,
+			    LENGTH_COL, (gint64) (slice.end - pos),
+			    START_STR_COL, start_str,
+			    END_STR_COL, end_str,
+			    LENGTH_STR_COL, length_str,
+			    -1);
+
+	g_free (length_str);
+	g_free (start_str);
+	g_free (end_str);
 
 	return TRUE;
 }
@@ -568,7 +620,7 @@ brasero_split_dialog_no_silence_message (BraseroSplitDialog *self)
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message),
 						  _("no silence could be retrieved."));
 
-	gtk_window_set_title (GTK_WINDOW (message), _("no silence"));
+	gtk_window_set_title (GTK_WINDOW (message), _("No Silence"));
 	gtk_dialog_run (GTK_DIALOG (message));
 	gtk_widget_destroy (message);
 }
@@ -671,11 +723,19 @@ brasero_split_dialog_clear_confirm_dialog (BraseroSplitDialog *self)
 					  GTK_DIALOG_DESTROY_WITH_PARENT|
 					  GTK_DIALOG_MODAL,
 					  GTK_MESSAGE_QUESTION,
-					  GTK_BUTTONS_YES_NO,
-					  _("This will remove all previous results."));
-	gtk_window_set_title (GTK_WINDOW (message), _("automatic split"));
+					  GTK_BUTTONS_NONE,
+					  _("Do you really want to carry on with automatic splitting?"));
+
+	gtk_window_set_title (GTK_WINDOW (message), _("Automatic Split"));
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (message),
-						  _("Do you want to carry on with automatic splitting nevertheless?"));
+						  _("This will remove all previous results."));
+
+	gtk_dialog_add_button (GTK_DIALOG (message),
+			       _("_Don't split"),
+			       GTK_RESPONSE_CANCEL);
+	gtk_dialog_add_button (GTK_DIALOG (message),
+			       _("_Split"),
+			       GTK_RESPONSE_YES);
 
 	answer = gtk_dialog_run (GTK_DIALOG (message));
 	gtk_widget_destroy (message);
