@@ -1,14 +1,12 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /*
  * brasero
- * Copyright (C) Philippe Rouquier 2007 <bonfire-app@wanadoo.fr>
+ * Copyright (C) Philippe Rouquier 2007-2008 <bonfire-app@wanadoo.fr>
  * 
- * brasero is free software.
- * 
- * You may redistribute it and/or modify it under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option)
- * any later version.
+ *  Brasero is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  * 
  * brasero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -161,6 +159,20 @@ brasero_time_button_frame_changed (GtkSpinButton *button,
 		       0);
 }
 
+void
+brasero_time_button_set_show_frames (BraseroTimeButton *self,
+				     gboolean show)
+{
+	BraseroTimeButtonPrivate *priv;
+
+	priv = BRASERO_TIME_BUTTON_PRIVATE (self);
+
+	if (show)
+		gtk_widget_show (priv->frame);
+	else
+		gtk_widget_hide (priv->frame);
+}
+
 void
 brasero_time_button_set_max (BraseroTimeButton *self,
 			     gint64 max)
