@@ -7,19 +7,21 @@
  ***************************************************************************/
 
 /*
- *  This program is free software; you can redistribute it and/or modify
+ *  Brasero is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Brasero is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -333,6 +335,14 @@ brasero_tool_dialog_set_valid (BraseroToolDialog *self,
 	gtk_widget_set_sensitive (self->priv->button, valid);
 }
 
+void
+brasero_tool_dialog_set_medium_type_shown (BraseroToolDialog *self,
+					   BraseroMediaType media_type)
+{
+	brasero_drive_selection_set_type_shown (BRASERO_DRIVE_SELECTION (self->priv->selector),
+						media_type);
+}
+
 BraseroMedium *
 brasero_tool_dialog_get_medium (BraseroToolDialog *self)
 {
@@ -489,6 +499,7 @@ brasero_tool_dialog_init (BraseroToolDialog *obj)
 
 	brasero_drive_selection_set_type_shown (BRASERO_DRIVE_SELECTION (obj->priv->selector),
 						BRASERO_MEDIA_TYPE_REWRITABLE|
+						BRASERO_MEDIA_TYPE_WRITABLE|
 						BRASERO_MEDIA_TYPE_READABLE);
 
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (obj)->vbox),
