@@ -1,14 +1,12 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /*
  * brasero
- * Copyright (C) Philippe Rouquier 2008 <bonfire-app@wanadoo.fr>
+ * Copyright (C) Philippe Rouquier 2005-2008 <bonfire-app@wanadoo.fr>
  * 
- * brasero is free software.
- * 
- * You may redistribute it and/or modify it under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option)
- * any later version.
+ *  Brasero is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  * 
  * brasero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -57,6 +55,7 @@ struct _BraseroDrivePrivate
 	BraseroMedium *medium;
 	BraseroDriveCaps caps;
 	gchar *path;
+	gchar *block_path;
 	gchar *udi;
 
 	gint bus;
@@ -290,6 +289,15 @@ brasero_drive_get_device (BraseroDrive *self)
 	return priv->path;
 }
 
+const gchar *
+brasero_drive_get_block_device (BraseroDrive *self)
+{
+	BraseroDrivePrivate *priv;
+
+	priv = BRASERO_DRIVE_PRIVATE (self);
+	return priv->block_path;
+}
+
 const gchar *
 brasero_drive_get_udi (BraseroDrive *self)
 {
@@ -392,6 +400,11 @@ brasero_drive_finalize (GObject *object)
 		priv->path = NULL;
 	}
 
+	if (priv->block_path) {
+		libhal_free_string (priv->block_path);
+		priv->block_path = NULL;
+	}
+
 	if (priv->udi) {
 		g_free (priv->udi);
 		priv->udi = NULL;
@@ -422,6 +435,12 @@ brasero_drive_init_real (BraseroDrive *drive)
 		priv->path = NULL;
 	}
 
+	priv->block_path = libhal_device_get_property_string (ctx, priv->udi, "block.device", NULL);
+	if (priv->block_path [0] == '\0') {
+		g_free (priv->block_path);
+		priv->block_path = NULL;
+	}
+
 	if (libhal_device_get_property_bool (ctx, priv->udi, "storage.cdrom.cdr", NULL))
 		priv->caps |= BRASERO_DRIVE_CAPS_CDR;
 	if (libhal_device_get_property_bool (ctx, priv->udi, "storage.cdrom.cdrw", NULL))
