@@ -1,14 +1,12 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /*
  * brasero
- * Copyright (C) Philippe Rouquier 2008 <bonfire-app@wanadoo.fr>
+ * Copyright (C) Philippe Rouquier 2005-2008 <bonfire-app@wanadoo.fr>
  * 
- * brasero is free software.
- * 
- * You may redistribute it and/or modify it under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option)
- * any later version.
+ *  Brasero is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  * 
  * brasero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -89,6 +87,9 @@ brasero_drive_get_display_name (BraseroDrive *drive);
 const gchar *
 brasero_drive_get_device (BraseroDrive *drive);
 
+const gchar *
+brasero_drive_get_block_device (BraseroDrive *drive);
+
 gboolean
 brasero_drive_get_bus_target_lun (BraseroDrive *drive,
 				  guint *bus,
