@@ -7,19 +7,21 @@
  ***************************************************************************/
 
 /*
- *  This program is free software; you can redistribute it and/or modify
+ *  Brasero is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Brasero is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 #ifndef JOB_H
@@ -147,6 +149,9 @@ brasero_job_get_session_output_size (BraseroJob *job, gint64 *blocks, gint64 *si
 BraseroBurnResult
 brasero_job_get_device (BraseroJob *job, gchar **device);
 
+BraseroBurnResult
+brasero_job_get_bus_target_lun (BraseroJob *job, gchar **BTL);
+
 BraseroBurnResult
 brasero_job_get_media (BraseroJob *job, BraseroMedia *media);
 
@@ -201,6 +206,20 @@ brasero_job_get_tmp_dir (BraseroJob *job,
 			 gchar **path,
 			 GError **error);
 
+/**
+ * Each tag can be retrieved by any job
+ */
+
+BraseroBurnResult
+brasero_job_tag_lookup (BraseroJob *job,
+			const gchar *tag,
+			GValue **value);
+
+BraseroBurnResult
+brasero_job_tag_add (BraseroJob *job,
+		     const gchar *tag,
+		     GValue *value);
+
 /**
  * Used to give job results and tell when a job has finished
  */
