@@ -3,12 +3,10 @@
  * brasero
  * Copyright (C) Philippe Rouquier 2007-2008 <bonfire-app@wanadoo.fr>
  * 
- * brasero is free software.
- * 
- * You may redistribute it and/or modify it under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option)
- * any later version.
+ *  Brasero is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  * 
  * brasero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -47,6 +45,7 @@
 #include "scsi-utils.h"
 #include "scsi-mode-pages.h"
 #include "scsi-status-page.h"
+#include "scsi-write-page.h"
 #include "scsi-q-subchannel.h"
 #include "scsi-dvd-structures.h"
 #include "burn-volume.h"
@@ -376,7 +375,7 @@ brasero_medium_get_max_write_speed (BraseroMedium *medium)
 	BraseroMediumPrivate *priv;
 
 	priv = BRASERO_MEDIUM_PRIVATE (medium);
-	return priv->max_wrt * 1024;
+	return priv->max_wrt * 1000;
 }
 
 gint64 *
@@ -392,8 +391,10 @@ brasero_medium_get_write_speeds (BraseroMedium *medium)
 	while (priv->wr_speeds [max] != 0) max ++;
 
 	speeds = g_new0 (gint64, max + 1);
+
+	/* NOTE: about the following, it's not KiB here but KB */
 	for (i = 0; i < max; i ++)
-		speeds [i] = priv->wr_speeds [i] * 1024;
+		speeds [i] = priv->wr_speeds [i] * 1000;
 
 	return speeds;
 }
@@ -873,6 +874,7 @@ brasero_medium_get_page_2A_max_speed (BraseroMedium *self,
 	g_free (data);
 	return BRASERO_BURN_OK;
 }
+ 
 /**
  * Functions to get information about disc contents
  */
@@ -900,13 +902,13 @@ brasero_medium_track_volume_size (BraseroMedium *self,
 	 * still add data (with growisofs). It is nevertheless on the 
 	 * condition that the fs is valid.
 	 * So we check if their first and only volume is valid. 
-	 * That's also used when the track size is reported a 300 Kio
+	 * That's also used when the track size is reported 300 KiB
 	 * see below */
 	vol = brasero_volume_source_open_device_handle (handle, NULL);
 	res = brasero_volume_get_size (vol,
 				       track->start,
 				       &nb_blocks,
-				       NULL);
+				       &error);
 	brasero_volume_source_close (vol);
 
 	if (!res) {
@@ -1021,7 +1023,7 @@ brasero_medium_track_get_info (BraseroMedium *self,
 
 	if (track->blocks_num <= 300) {
 		/* Now here is a potential bug: we can write tracks (data or
-		 * not) shorter than 300 Kio /2 sec but they will be padded to
+		 * not) shorter than 300 KiB /2 sec but they will be padded to
 		 * reach this floor value. It means that blocks_num is always
 		 * 300 blocks even if the data length on the track is actually
 		 * shorter.
@@ -1094,17 +1096,8 @@ brasero_medium_track_get_info (BraseroMedium *self,
 		else
 			BRASERO_BURN_LOG ("Detected runouts");
 	}
-	else if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_PLUS)
-	     ||  BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_PLUS_DL)
-	     ||  BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_RESTRICTED)) {
-		BRASERO_BURN_LOG ("DVD+RW (DL) or DVD-RW (restricted overwrite) checking volume size");
-		brasero_medium_track_volume_size (self, track, handle);
-	}
-
-
-	if (track_info.next_wrt_address_valid)
-		priv->next_wr_add = BRASERO_GET_32 (track_info.next_wrt_address);
 
+	/* NOTE: DVD+RW, DVD-RW (restricted overwrite) never reach this function */
 	BRASERO_BURN_LOG ("Track %i (session %i): type = %i start = %llu size = %llu",
 			  track_num,
 			  track->session,
@@ -1115,6 +1108,94 @@ brasero_medium_track_get_info (BraseroMedium *self,
 	return BRASERO_BURN_OK;
 }
 
+static BraseroBurnResult
+brasero_medium_track_get_nwa (BraseroMedium *self,
+			      BraseroDeviceHandle *handle,
+			      BraseroScsiErrCode *code)
+{
+	BraseroScsiModeData *data = NULL;
+	BraseroScsiTrackInfo track_info;
+	BraseroScsiWritePage *wrt_page;
+	BraseroMediumPrivate *priv;
+	BraseroScsiResult result;
+	gint track_num;
+	int size;
+
+	BRASERO_BURN_LOG ("Retrieving NWA");
+
+	priv = BRASERO_MEDIUM_PRIVATE (self);
+
+	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_PLUS)
+	||  BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_RESTRICTED)) {
+		BRASERO_BURN_LOG ("Overwritable medium  => skipping");
+		return BRASERO_BURN_OK;
+	}
+
+	/* make sure the current write mode is TAO. Otherwise the drive will
+	 * return the first sector of the pregap instead of the first user
+	 * accessible sector. */
+	result = brasero_spc1_mode_sense_get_page (handle,
+						   BRASERO_SPC_PAGE_WRITE,
+						   &data,
+						   &size,
+						   code);
+	if (result != BRASERO_SCSI_OK) {
+		g_free (data);
+
+		BRASERO_BURN_LOG ("MODE SENSE failed");
+		return BRASERO_BURN_ERR;
+	}
+
+	wrt_page = (BraseroScsiWritePage *) &data->page;
+	wrt_page->write_type = BRASERO_SCSI_WRITE_TAO;
+
+	result = brasero_spc1_mode_select (handle, data, size, code);
+	g_free (data);
+
+	if (result != BRASERO_SCSI_OK) {
+		BRASERO_BURN_LOG ("MODE SELECT failed");
+		return BRASERO_BURN_ERR;
+	}
+
+	/* at this point we know the type of the disc that's why we set the 
+	 * size according to this type. That may help to avoid outrange address
+	 * errors. */
+	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVD_DL|BRASERO_MEDIUM_WRITABLE))
+		size = 48;
+	else if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_PLUS|BRASERO_MEDIUM_WRITABLE))
+		size = 40;
+	else
+		size = 36;
+
+	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_CDR)
+	||  BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_CDRW)
+	/* The following includes DL */
+	||  BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDR_PLUS)) 
+		track_num = 0xFF;
+	else
+		track_num = g_slist_length (priv->tracks);
+
+	result = brasero_mmc1_read_track_info (handle,
+					       track_num,
+					       &track_info,
+					       &size,
+					       code);
+
+	if (result != BRASERO_SCSI_OK) {
+		BRASERO_BURN_LOG ("READ TRACK INFO failed");
+		return BRASERO_BURN_ERR;
+	}
+
+	if (track_info.next_wrt_address_valid) {
+		priv->next_wr_add = BRASERO_GET_32 (track_info.next_wrt_address);
+		BRASERO_BURN_LOG ("Next Writable Address is %d", priv->next_wr_add);
+	}
+	else
+		BRASERO_BURN_LOG ("No Next Writable Address");
+
+	return BRASERO_BURN_OK;
+}
+
 /**
  * NOTE: for DVD-R multisession we lose 28688 blocks for each session
  * so the capacity is the addition of all session sizes + 28688 for each
@@ -1123,18 +1204,33 @@ brasero_medium_track_get_info (BraseroMedium *self,
  */
 
 static void
-brasero_medium_add_DVD_plus_RW_leadout (BraseroMedium *self,
-					gint32 start)
+brasero_medium_add_DVD_plus_RW_leadout (BraseroMedium *self)
 {
 	BraseroMediumTrack *leadout;
 	BraseroMediumPrivate *priv;
+	gint64 blocks_num;
+	gint32 start;
 
 	priv = BRASERO_MEDIUM_PRIVATE (self);
 
+	/* determine the start */
+	if (priv->tracks) {
+		BraseroMediumTrack *track;
+
+		track = priv->tracks->data;
+		start = track->start + track->blocks_num;
+		blocks_num = priv->block_num - ((track->blocks_num > 300) ? track->blocks_num : 300);
+	}
+	else {
+		start = 0;
+		blocks_num = priv->block_num;
+	}
+
 	leadout = g_new0 (BraseroMediumTrack, 1);
 	priv->tracks = g_slist_append (priv->tracks, leadout);
 
 	leadout->start = start;
+	leadout->blocks_num = blocks_num;
 	leadout->type = BRASERO_MEDIUM_TRACK_LEADOUT;
 
 	/* we fabricate the leadout here. We don't really need one in 
@@ -1144,13 +1240,6 @@ brasero_medium_add_DVD_plus_RW_leadout (BraseroMedium *self,
 	 * buggy */
 	priv->next_wr_add = 0;
 
-	leadout->blocks_num = priv->block_num;
-	if (g_slist_length (priv->tracks) > 1) {
-		BraseroMediumTrack *track;
-
-		track = priv->tracks->data;
-		leadout->blocks_num -= ((track->blocks_num > 300) ? track->blocks_num : 300);
-	}
 	BRASERO_BURN_LOG ("Adding fabricated leadout start = %llu length = %llu",
 			  leadout->start,
 			  leadout->blocks_num);
@@ -1224,22 +1313,16 @@ brasero_medium_get_sessions_info (BraseroMedium *self,
 				track->type |= BRASERO_MEDIUM_TRACK_INCREMENTAL;
 		}
 
-		brasero_medium_track_get_info (self,
-					       multisession,
-					       track,
-					       g_slist_length (priv->tracks),
-					       handle,
-					       code);
-
-		if (desc->control & BRASERO_SCSI_TRACK_COPY)
-			track->type |= BRASERO_MEDIUM_TRACK_COPY;
-
 		if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_PLUS)
 		||  BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_RESTRICTED)) {
 			BraseroBurnResult result;
 
-			/* a special case for these two kinds of media (DVD+RW)
-			 * which have only one track: the first. */
+			/* A special case for these two kinds of media (DVD+RW)
+			 * which have only one track: the first. Since it's not
+			 * possible to know the amount of data that were really
+			 * written in this session, read the filesystem. */
+			BRASERO_BURN_LOG ("DVD+RW (DL) or DVD-RW (restricted overwrite) checking volume size");
+			track->session = 1;
 			result = brasero_medium_track_volume_size (self, 
 								   track,
 								   handle);
@@ -1253,26 +1336,33 @@ brasero_medium_get_sessions_info (BraseroMedium *self,
 
 				BRASERO_BURN_LOG ("Empty first session.");
 			}
-			else
+			else {
 				priv->next_wr_add = 0;
+				BRASERO_BURN_LOG ("Track 1 (session %i): type = %i start = %llu size = %llu",
+						  track->session,
+						  track->type,
+						  track->start,
+						  track->blocks_num);
+			}
+
+			/* NOTE: the next track should be the leadout */
+			continue;
 		}
+
+		brasero_medium_track_get_info (self,
+					       multisession,
+					       track,
+					       g_slist_length (priv->tracks),
+					       handle,
+					       code);
 	}
 
 	/* put the tracks in the right order */
 	priv->tracks = g_slist_reverse (priv->tracks);
 
 	if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_PLUS)
-	||  BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_RESTRICTED)) {
-		gint32 start;
-
-		/* It starts where the other one finishes */
-		if (priv->tracks)
-			start = BRASERO_GET_32 (desc->track_start);
-		else
-			start = 0;
-
-		brasero_medium_add_DVD_plus_RW_leadout (self, start);
-	}
+	||  BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_RESTRICTED))
+		brasero_medium_add_DVD_plus_RW_leadout (self);
 	else if (!(priv->info & BRASERO_MEDIUM_CLOSED)) {
 		BraseroMediumTrack *track;
 
@@ -1289,6 +1379,8 @@ brasero_medium_get_sessions_info (BraseroMedium *self,
 					       g_slist_length (priv->tracks),
 					       handle,
 					       code);
+
+		brasero_medium_track_get_nwa (self, handle, code);
 	}
 
 	g_free (toc);
@@ -1334,7 +1426,7 @@ brasero_medium_get_contents (BraseroMedium *self,
 
 		if (BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_PLUS)
 		||  BRASERO_MEDIUM_IS (priv->info, BRASERO_MEDIUM_DVDRW_RESTRICTED))
-			brasero_medium_add_DVD_plus_RW_leadout (self, 0);
+			brasero_medium_add_DVD_plus_RW_leadout (self);
 		else {
 			track = g_new0 (BraseroMediumTrack, 1);
 			track->start = 0;
@@ -1368,8 +1460,6 @@ end:
 	return result;
 }
 
-#if 0
-
 /**
  * These are special routines for old CD-R(W) drives that don't conform to MMC
  */
@@ -1736,8 +1826,6 @@ brasero_medium_get_CD_sessions_info (BraseroMedium *self,
 	return BRASERO_BURN_OK;
 }
 
-#endif
-
 static BraseroBurnResult
 brasero_medium_old_drive_get_disc_info (BraseroMedium *self,
 					BraseroDeviceHandle *handle,
@@ -1791,7 +1879,7 @@ brasero_medium_old_drive_get_disc_info (BraseroMedium *self,
 	}
 
 	/* get the contents */
-	result = brasero_medium_get_sessions_info (self, handle, code);
+	result = brasero_medium_get_CD_sessions_info (self, handle, code);
 	return result;
 }
 
@@ -2218,6 +2306,7 @@ brasero_medium_try_open (BraseroMedium *self)
 	 * but we re-try to open it every second */
 	BRASERO_BURN_LOG ("Trying to open device %s", path);
 	handle = brasero_device_handle_open (path, &code);
+
 	if (!handle) {
 		if (code == BRASERO_SCSI_NOT_READY) {
 			BRASERO_BURN_LOG ("Device busy");
