@@ -2,12 +2,10 @@
  * brasero
  * Copyright (C) Philippe Rouquier 2007-2008 <bonfire-app@wanadoo.fr>
  * 
- * brasero is free software.
- * 
- * You may redistribute it and/or modify it under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option)
- * any later version.
+ *  Brasero is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
  * 
  * brasero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -605,6 +603,7 @@ brasero_plugin_check_image_flags (BraseroPlugin *self,
 	BraseroPluginPrivate *priv;
 
 	priv = BRASERO_PLUGIN_PRIVATE (self);
+
 	current &= BRASERO_PLUGIN_IMAGE_FLAG_MASK;
 
 	/* If there is no flag that's no use checking anything. If there is no
@@ -622,6 +621,21 @@ brasero_plugin_check_image_flags (BraseroPlugin *self,
 					     NULL);
 }
 
+gboolean
+brasero_plugin_check_media_restrictions (BraseroPlugin *self,
+					 BraseroMedia media)
+{
+	BraseroPluginPrivate *priv;
+
+	priv = BRASERO_PLUGIN_PRIVATE (self);
+
+	/* no restrictions */
+	if (!priv->flags)
+		return TRUE;
+
+	return (brasero_plugin_get_flags (priv->flags, media) != NULL);
+}
+
 gboolean
 brasero_plugin_get_record_flags (BraseroPlugin *self,
 				 BraseroMedia media,
