@@ -7,19 +7,21 @@
  ***************************************************************************/
 
 /*
- *  This program is free software; you can redistribute it and/or modify
+ *  Brasero is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Brasero is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -285,11 +287,13 @@ brasero_process_finished (BraseroProcess *self)
 		g_free (uri);
 	}
 
-	brasero_job_add_track (BRASERO_JOB (self), track);
+	if (track) {
+		brasero_job_add_track (BRASERO_JOB (self), track);
 
-	/* It's good practice to unref the track afterwards as we don't need it
-	 * anymore. BraseroTaskCtx refs it. */
-	brasero_track_unref (track);
+		/* It's good practice to unref the track afterwards as we don't
+		 * need it anymore. BraseroTaskCtx refs it. */
+		brasero_track_unref (track);
+	}
 
 	klass->post (BRASERO_JOB (self));
 	return BRASERO_BURN_OK;
