@@ -7,19 +7,21 @@
  ***************************************************************************/
 
 /*
- *  This program is free software; you can redistribute it and/or modify
+ *  Brasero is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Brasero is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -370,7 +372,6 @@ brasero_burn_session_dest_media_removed (BraseroDrive *drive,
 					 BraseroMedium *medium,
 					 BraseroBurnSession *self)
 {
-	
 	g_signal_emit (self,
 		       brasero_burn_session_signals [OUTPUT_CHANGED_SIGNAL],
 		       0);
@@ -505,9 +506,9 @@ brasero_burn_session_get_num_copies (BraseroBurnSession *self)
 }
 
 static gchar *
-brasero_burn_session_get_file_complement (BraseroBurnSession *self,
-					  BraseroImageFormat format,
-					  const gchar *path)
+brasero_burn_session_get_image_complement (BraseroBurnSession *self,
+					   BraseroImageFormat format,
+					   const gchar *path)
 {
 	gchar *retval = NULL;
 	BraseroBurnSessionPrivate *priv;
@@ -515,28 +516,28 @@ brasero_burn_session_get_file_complement (BraseroBurnSession *self,
 	priv = BRASERO_BURN_SESSION_PRIVATE (self);
 
 	if (format == BRASERO_IMAGE_FORMAT_CLONE) {
-		if (g_str_has_suffix (path, ".toc"))
-			retval = g_strdup_printf ("%.*sraw",
+		if (g_str_has_suffix (path, ".raw"))
+			retval = g_strdup_printf ("%.*stoc",
 						  strlen (path) - 3,
 						  path);
 		else
-			retval = g_strdup_printf ("%s.raw", path);
+			retval = g_strdup_printf ("%s.toc", path);
 	}
 	else if (format == BRASERO_IMAGE_FORMAT_CUE) {
-		if (g_str_has_suffix (path, ".cue"))
-			retval = g_strdup_printf ("%.*sbin",
+		if (g_str_has_suffix (path, ".bin"))
+			retval = g_strdup_printf ("%.*scue",
 						  strlen (path) - 3,
 						  path);
 		else
 			retval = g_strdup_printf ("%s.bin", path);
 	}
 	else if (format == BRASERO_IMAGE_FORMAT_CDRDAO) {
-		if (g_str_has_suffix (path, ".toc"))
-			retval = g_strdup_printf ("%.*sbin",
+		if (g_str_has_suffix (path, ".bin"))
+			retval = g_strdup_printf ("%.*stoc",
 						  strlen (path) - 3,
 						  path);
 		else
-			retval = g_strdup_printf ("%s.bin", path);
+			retval = g_strdup_printf ("%s.toc", path);
 	}
 	else
 		retval = NULL;
@@ -657,40 +658,69 @@ brasero_burn_session_get_output (BraseroBurnSession *self,
 		return BRASERO_BURN_ERR;
 	}
 
-	/* output paths were set so test them and returns them if OK */
-	if (priv->settings->image) {
-		result = brasero_burn_session_file_test (self,
-							 priv->settings->image,
-							 error);
-		if (result != BRASERO_BURN_OK) {
-			BRASERO_BURN_LOG ("Problem with image existence");
-			return result;
+	if (image) {
+		/* output paths were set so test them and returns them if OK */
+		if (priv->settings->image) {
+			result = brasero_burn_session_file_test (self,
+								 priv->settings->image,
+								 error);
+			if (result != BRASERO_BURN_OK) {
+				BRASERO_BURN_LOG ("Problem with image existence");
+				return result;
+			}
+
+			*image = g_strdup (priv->settings->image);
 		}
-	}
-	else {
-		BRASERO_BURN_LOG ("no output specified");
+		else if (priv->settings->toc) {
+			gchar *complement;
 
-		g_set_error (error,
-			     BRASERO_BURN_ERROR,
-			     BRASERO_BURN_ERROR_GENERAL,
-			     _("no output specified"));
-		return BRASERO_BURN_ERR;
-	}
+			/* get the cuesheet complement */
+			complement = brasero_image_format_get_complement (priv->settings->format,
+									  priv->settings->toc);
+			if (!complement) {
+				BRASERO_BURN_LOG ("no output specified");
 
-	if (priv->settings->toc) {
-		result = brasero_burn_session_file_test (self,
-							 priv->settings->toc,
-							 error);
-		if (result != BRASERO_BURN_OK) {
-			BRASERO_BURN_LOG ("Problem with toc existence");
-			return result;
+				g_set_error (error,
+					     BRASERO_BURN_ERROR,
+					     BRASERO_BURN_ERROR_GENERAL,
+					     _("no output specified"));
+				return BRASERO_BURN_ERR;
+			}
+
+			result = brasero_burn_session_file_test (self,
+								 complement,
+								 error);
+			if (result != BRASERO_BURN_OK) {
+				BRASERO_BURN_LOG ("Problem with image existence");
+				return result;
+			}
+
+			*image = complement;
+		}
+		else {
+			BRASERO_BURN_LOG ("no output specified");
+
+			g_set_error (error,
+				     BRASERO_BURN_ERROR,
+				     BRASERO_BURN_ERROR_GENERAL,
+				     _("no output specified"));
+			return BRASERO_BURN_ERR;
 		}
 	}
 
-	if (image)
-		*image = g_strdup (priv->settings->image);
-	if (toc)
+	if (toc) {
+		if (priv->settings->toc) {
+			result = brasero_burn_session_file_test (self,
+								 priv->settings->toc,
+								 error);
+			if (result != BRASERO_BURN_OK) {
+				BRASERO_BURN_LOG ("Problem with toc existence");
+				return result;
+			}
+		}
+
 		*toc = g_strdup (priv->settings->toc);
+	}
 
 	return BRASERO_BURN_OK;
 }
@@ -769,43 +799,6 @@ brasero_burn_session_set_image_output_full (BraseroBurnSession *self,
 	return BRASERO_BURN_OK;
 }
 
-BraseroBurnResult
-brasero_burn_session_set_image_output (BraseroBurnSession *self,
-				       BraseroImageFormat format,
-				       const gchar *path)
-{
-	BraseroBurnSessionPrivate *priv;
-	BraseroBurnResult result;
-	gchar *complement;
-	gchar *image = NULL;
-	gchar *toc = NULL;
-
-	g_return_val_if_fail (BRASERO_IS_BURN_SESSION (self), BRASERO_BURN_ERR);
-
-	priv = BRASERO_BURN_SESSION_PRIVATE (self);
-
-	/* find the file complement */
-	complement = brasero_burn_session_get_file_complement (self,
-							       format,
-							       path);
-
-	brasero_burn_session_set_image_output_retval (self,
-						      format,
-						      &image,
-						      &toc,
-						      g_strdup (path),
-						      complement);
-
-	result = brasero_burn_session_set_image_output_full (self,
-							     format,
-							     image,
-							     toc);
-	g_free (image);
-	g_free (toc);
-
-	return result;
-}
-
 /**
  *
  */
@@ -956,7 +949,7 @@ brasero_burn_session_get_tmp_image (BraseroBurnSession *self,
 
 	if (format != BRASERO_IMAGE_FORMAT_BIN) {
 		/* toc tmp file */
-		complement = brasero_burn_session_get_file_complement (self, format, path);
+		complement = brasero_burn_session_get_image_complement (self, format, path);
 		if (complement) {
 			result = brasero_burn_session_file_test (self,
 								 complement,
@@ -1094,6 +1087,22 @@ brasero_burn_session_tag_value_free (gpointer user_data)
 	g_free (value);
 }
 
+BraseroBurnResult
+brasero_burn_session_tag_remove (BraseroBurnSession *self,
+				 const gchar *tag)
+{
+	BraseroBurnSessionPrivate *priv;
+
+	g_return_val_if_fail (BRASERO_IS_BURN_SESSION (self), BRASERO_BURN_ERR);
+
+	priv = BRASERO_BURN_SESSION_PRIVATE (self);
+	if (!priv->tags)
+		return BRASERO_BURN_ERR;
+
+	g_hash_table_remove (priv->tags, tag);
+	return BRASERO_BURN_OK;
+}
+
 BraseroBurnResult
 brasero_burn_session_tag_add (BraseroBurnSession *self,
 			      const gchar *tag,
@@ -1438,10 +1447,7 @@ brasero_burn_session_logv (BraseroBurnSession *self,
 	if (!priv->session)
 		return;
 
-	if (arg_list)
-		message = g_strdup_vprintf (format, arg_list);
-	else
-		message = g_strdup (format);
+	message = g_strdup_vprintf (format, arg_list);
 
 	/* we also need to validate the messages to be in UTF-8 */
 	if (!g_utf8_validate (message, -1, (const gchar**) &offending))
@@ -1603,6 +1609,9 @@ brasero_burn_session_get_config_key (BraseroBurnSession *self,
 
 	g_strdelimit (disc_type, " +()", '_');
 
+	display_name = display_name ? display_name : "";
+	disc_type = disc_type ? disc_type : "";
+
 	switch (brasero_burn_session_get_input_type (self, NULL)) {
 	case BRASERO_TRACK_TYPE_NONE:
 		key = g_strdup_printf ("%s/%s/none_%s/%s",
