@@ -3,16 +3,16 @@
  *
  *  Sun Nov 26 19:20:31 2006
  *  Copyright  2006  Rouquier Philippe
- *  <Rouquier Philippe@localhost.localdomain>
+ *  <bonfire-app@wanadoo.fr>
  ****************************************************************************/
 
 /*
- * This program is free software; you can redistribute it and/or modify
+ * Brasero is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  * 
- * This program is distributed in the hope that it will be useful,
+ * Brasero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU Library General Public License for more details.
@@ -27,10 +27,7 @@
 #ifndef _BURN_SUSP_H
 #define _BURN_SUSP_H
 
-#ifdef __cplusplus
-extern "C"
-{
-#endif
+G_BEGIN_DECLS
 
 struct _BraseroSuspCtx {
 	gchar *rr_name;
@@ -52,9 +49,7 @@ brasero_susp_ctx_clean (BraseroSuspCtx *ctx);
 gboolean
 brasero_susp_read (BraseroSuspCtx *ctx, gchar *buffer, gint max);
 
-#ifdef __cplusplus
-}
-#endif
+G_END_DECLS
 
 #endif /* _BURN_SUSP_H */
 
