@@ -2,17 +2,17 @@
  *            burn-track.c
  *
  *  Thu Dec  7 09:50:38 2006
- *  Copyright  2006  algernon
- *  <algernon@localhost.localdomain>
+ *  Copyright  2006  Rouquier Philippe
+ *  <bonfire-app@wanadoo.fr>
  ****************************************************************************/
 
 /*
- * This program is free software; you can redistribute it and/or modify
+ * Brasero is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  * 
- * This program is distributed in the hope that it will be useful,
+ * Brasero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU Library General Public License for more details.
@@ -943,7 +943,7 @@ brasero_track_set_checksum (BraseroTrack *track,
 const gchar *
 brasero_track_get_checksum (BraseroTrack *track)
 {
-	return track->checksum;
+	return track->checksum ? track->checksum : "";
 }
 
 BraseroChecksumType
