@@ -1,26 +1,23 @@
 /* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /*
  * brasero
- * Copyright (C) Philippe Rouquier 2008 <bonfire-app@wanadoo.fr>
+ * Copyright (C) Philippe Rouquier 2005-2008 <bonfire-app@wanadoo.fr>
+ *
+ * Brasero is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  * 
- * brasero is free software.
- * 
- * You may redistribute it and/or modify it under the terms of the
- * GNU General Public License, as published by the Free Software
- * Foundation; either version 2 of the License, or (at your option)
- * any later version.
- * 
- * brasero is distributed in the hope that it will be useful,
+ * Brasero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
- * See the GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Library General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with brasero.  If not, write to:
- * 	The Free Software Foundation, Inc.,
- * 	51 Franklin Street, Fifth Floor
- * 	Boston, MA  02110-1301, USA.
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
  */
+
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
@@ -75,7 +72,12 @@ brasero_volume_get_gvolume (BraseroVolume *self)
 	priv = BRASERO_VOLUME_PRIVATE (self);
 
 	drive = brasero_medium_get_drive (BRASERO_MEDIUM (self));
+
+#if defined(HAVE_STRUCT_USCSI_CMD)
+	volume_path = brasero_drive_get_block_device (drive);
+#else
 	volume_path = brasero_drive_get_device (drive);
+#endif
 
 	/* NOTE: medium-monitor already holds a reference for GVolumeMonitor */
 	monitor = g_volume_monitor_get ();
@@ -88,6 +90,7 @@ brasero_volume_get_gvolume (BraseroVolume *self)
 
 		tmp = iter->data;
 		device_path = g_volume_get_identifier (tmp, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
+		BRASERO_BURN_LOG ("Found volume %s", device_path);
 		if (!strcmp (device_path, volume_path)) {
 			volume = tmp;
 			g_free (device_path);
@@ -458,9 +461,14 @@ brasero_volume_eject_finish (GObject *source,
 	BraseroVolumePrivate *priv;
 
 	priv = BRASERO_VOLUME_PRIVATE (self);
-	priv->result = g_drive_eject_finish (G_DRIVE (source),
-					     result,
-					     &priv->error);
+	if (G_IS_DRIVE (source))
+		priv->result = g_drive_eject_finish (G_DRIVE (source),
+						     result,
+						     &priv->error);
+	else
+		priv->result = g_volume_eject_finish (G_VOLUME (source),
+						      result,
+						      &priv->error);
 
 	if (priv->error) {
 		if (priv->error->code == G_IO_ERROR_FAILED_HANDLED) {
@@ -476,6 +484,56 @@ brasero_volume_eject_finish (GObject *source,
 		brasero_volume_operation_end (self);
 }
 
+gboolean
+brasero_volume_eject_gvolume (BraseroVolume *self,
+			      gboolean wait,
+			      GVolume *volume,
+			      GError **error)
+{
+	BraseroVolumePrivate *priv;
+	gboolean result;
+
+	priv = BRASERO_VOLUME_PRIVATE (self);
+
+	if (!g_volume_can_eject (volume))
+		return FALSE;
+
+	if (wait) {
+		gulong eject_sig;
+		BraseroDrive *drive;
+
+		drive = brasero_medium_get_drive (BRASERO_MEDIUM (self));
+		eject_sig = g_signal_connect (drive,
+					      "medium-removed",
+					      G_CALLBACK (brasero_volume_ejected_cb),
+					      self);
+
+		g_volume_eject (volume,
+				G_MOUNT_UNMOUNT_NONE,
+				priv->cancel,
+				brasero_volume_eject_finish,
+				self);
+
+		g_object_ref (self);
+		result = brasero_volume_wait_for_operation_end (self, error);
+		g_object_unref (self);
+
+		/* NOTE: from this point on self is no longer valid */
+
+		g_signal_handler_disconnect (drive, eject_sig);
+	}
+	else {
+		g_volume_eject (volume,
+				G_MOUNT_UNMOUNT_NONE,
+				priv->cancel,
+				NULL,
+				self);
+		result = TRUE;
+	}
+
+	return result;
+}
+
 gboolean
 brasero_volume_eject (BraseroVolume *self,
 		      gboolean wait,
@@ -496,11 +554,17 @@ brasero_volume_eject (BraseroVolume *self,
 		return FALSE;
 
 	gdrive = g_volume_get_drive (volume);
-	g_object_unref (volume);
+	if (!gdrive) {
+		result = brasero_volume_eject_gvolume (self, wait, volume, error);
+		g_object_unref (volume);
+		return result;
+	}
 
 	if (!g_drive_can_eject (gdrive)) {
+		result = brasero_volume_eject_gvolume (self, wait, volume, error);
+		g_object_unref (volume);
 		g_object_unref (gdrive);
-		return FALSE;
+		return result;
 	}
 
 	if (wait) {
