@@ -7,19 +7,21 @@
  ***************************************************************************/
 
 /*
- *  This program is free software; you can redistribute it and/or modify
+ *  Brasero is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Brasero is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -374,18 +376,19 @@ brasero_burn_eject_src_media (BraseroBurn *self,
 		}
 	}
 
-	brasero_volume_eject (BRASERO_VOLUME (medium), TRUE, NULL);
+	brasero_volume_eject (BRASERO_VOLUME (medium), TRUE, error);
 	medium = brasero_drive_get_medium (priv->src);
 	if (medium && brasero_medium_get_status (medium) != BRASERO_MEDIUM_NONE) {
 		gchar *name;
 
 		name = brasero_drive_get_display_name (priv->src);
 
-		g_set_error (error,
-			     BRASERO_BURN_ERROR,
-			     BRASERO_BURN_ERROR_GENERAL,
-			     _("the media in %s can't be ejected"),
-			     name);
+		if (error && !(*error))
+			g_set_error (error,
+				     BRASERO_BURN_ERROR,
+				     BRASERO_BURN_ERROR_GENERAL,
+				     _("the media in %s can't be ejected"),
+				     name);
 
 		g_free (name);
 
@@ -876,7 +879,10 @@ again:
 		/* NOTE: no need to error out here since the only thing
 		 * we are interested in is if it is AUDIO or not or if
 		 * the disc we are copying has audio tracks only or not */
-		if (input.type == BRASERO_TRACK_TYPE_AUDIO) {
+		if (input.type == BRASERO_TRACK_TYPE_AUDIO
+		&& !(input.subtype.audio_format & (BRASERO_VIDEO_FORMAT_UNDEFINED|
+						   BRASERO_VIDEO_FORMAT_VCD|
+						   BRASERO_VIDEO_FORMAT_VIDEO_DVD))) {
 			result = brasero_burn_emit_signal (burn, WARN_REWRITABLE_SIGNAL, BRASERO_BURN_CANCEL);
 			if (result != BRASERO_BURN_OK)
 				goto end;
@@ -1063,7 +1069,8 @@ again:
 }
 
 static BraseroBurnResult
-brasero_burn_unlock_src_media (BraseroBurn *burn)
+brasero_burn_unlock_src_media (BraseroBurn *burn,
+			       GError **error)
 {
 	BraseroBurnPrivate *priv = BRASERO_BURN_PRIVATE (burn);
 	BraseroMedium *medium;
@@ -1078,7 +1085,7 @@ brasero_burn_unlock_src_media (BraseroBurn *burn)
 
 	medium = brasero_drive_get_medium (priv->src);
 	if (priv->mounted_by_us) {
-		brasero_volume_umount (BRASERO_VOLUME (medium), TRUE, NULL);
+		brasero_volume_umount (BRASERO_VOLUME (medium), TRUE, error);
 		priv->mounted_by_us = 0;
 	}
 
@@ -1086,14 +1093,15 @@ brasero_burn_unlock_src_media (BraseroBurn *burn)
 	brasero_drive_unlock (priv->src);
 
 	if (BRASERO_BURN_SESSION_EJECT (priv->session))
-		brasero_volume_eject (BRASERO_VOLUME (medium), FALSE, NULL);
+		brasero_volume_eject (BRASERO_VOLUME (medium), FALSE, error);
 
 	priv->src = NULL;
 	return BRASERO_BURN_OK;
 }
 
 static BraseroBurnResult
-brasero_burn_unlock_dest_media (BraseroBurn *burn)
+brasero_burn_unlock_dest_media (BraseroBurn *burn,
+				GError **error)
 {
 	BraseroBurnPrivate *priv = BRASERO_BURN_PRIVATE (burn);
 	BraseroMedium *medium;
@@ -1111,17 +1119,18 @@ brasero_burn_unlock_dest_media (BraseroBurn *burn)
 	medium = brasero_drive_get_medium (priv->dest);
 
 	if (BRASERO_BURN_SESSION_EJECT (priv->session))
-		brasero_volume_eject (BRASERO_VOLUME (medium), FALSE, NULL);
+		brasero_volume_eject (BRASERO_VOLUME (medium), FALSE, error);
 
 	priv->dest = NULL;
 	return BRASERO_BURN_OK;
 }
 
 static BraseroBurnResult
-brasero_burn_unlock_medias (BraseroBurn *burn)
+brasero_burn_unlock_medias (BraseroBurn *burn,
+			    GError **error)
 {
-	brasero_burn_unlock_dest_media (burn);
-	brasero_burn_unlock_src_media (burn);
+	brasero_burn_unlock_dest_media (burn, error);
+	brasero_burn_unlock_src_media (burn, error);
 
 	return BRASERO_BURN_OK;
 }
@@ -2164,14 +2173,17 @@ brasero_burn_check (BraseroBurn *self,
 
 	result = brasero_burn_check_real (self, track, error);
 
-	brasero_burn_unlock_medias (self);
-
 	brasero_burn_powermanagement (self, FALSE);
 
 	/* no need to check the result of the comparison, it's set in session */
 	priv->session = NULL;
 	g_object_unref (session);
 
+	if (result == BRASERO_BURN_OK)
+		result = brasero_burn_unlock_medias (self, error);
+	else
+		brasero_burn_unlock_medias (self, NULL);
+
 	return result;
 }
 
@@ -2241,7 +2253,7 @@ brasero_burn_same_src_dest (BraseroBurn *self,
 	/* run */
 	result = brasero_burn_record_session (self, TRUE, error);
 	if (result != BRASERO_BURN_OK) {
-		brasero_burn_unlock_src_media (self);
+		brasero_burn_unlock_src_media (self, NULL);
 		goto end;
 	}
 
@@ -2336,7 +2348,7 @@ brasero_burn_record (BraseroBurn *burn,
 
 			/* see if we still need it to be locked */
 			if (brasero_burn_session_get_input_type (session, NULL) != BRASERO_TRACK_TYPE_DISC)
-				brasero_burn_unlock_src_media (burn);
+				brasero_burn_unlock_src_media (burn, NULL);
 
 			result = brasero_burn_record_session (burn, TRUE, error);
 			if (result != BRASERO_BURN_OK)
@@ -2346,7 +2358,10 @@ brasero_burn_record (BraseroBurn *burn,
 
 end:
 
-	brasero_burn_unlock_medias (burn);
+	if (result == BRASERO_BURN_OK)
+		result = brasero_burn_unlock_medias (burn, error);
+	else
+		brasero_burn_unlock_medias (burn, NULL);
 
 	if (error && (*error) == NULL
 	&& (result == BRASERO_BURN_NOT_READY
@@ -2466,12 +2481,14 @@ end:
 	if (ret_error)
 		g_propagate_error (error, ret_error);
 
-	brasero_burn_unlock_medias (burn);
+	if (result == BRASERO_BURN_OK && !ret_error)
+		result = brasero_burn_unlock_medias (burn, error);
+	else
+		brasero_burn_unlock_medias (burn, NULL);
 
 	if (result == BRASERO_BURN_OK)
 		brasero_burn_action_changed_real (burn, BRASERO_BURN_ACTION_FINISHED);
 
-
 	brasero_burn_powermanagement (burn, FALSE);
 
 	/* release session */
