@@ -7,19 +7,21 @@
  ***************************************************************************/
 
 /*
- *  This program is free software; you can redistribute it and/or modify
+ *  Brasero is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Brasero is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -68,7 +70,7 @@ brasero_cdrdao_read_stderr_image (BraseroCdrdao *cdrdao, const gchar *line)
 		brasero_job_get_action (BRASERO_JOB (cdrdao), &action);
 		if (action == BRASERO_JOB_ACTION_SIZE) {
 			/* get the number of sectors. As we added -raw sector = 2352 bytes */
-			brasero_job_set_output_size_for_current_track (BRASERO_JOB (cdrdao), s1, s1 * 2352);
+			brasero_job_set_output_size_for_current_track (BRASERO_JOB (cdrdao), s1, (gint64) s1 * 2352ULL);
 			brasero_job_finished_session (BRASERO_JOB (cdrdao));
 		}
 	}
@@ -493,7 +495,7 @@ brasero_cdrdao_set_argv (BraseroProcess *process,
 
 			brasero_job_set_output_size_for_current_track (BRASERO_JOB (cdrdao),
 								       sectors,
-								       sectors * 2352);
+								       sectors * 2352ULL);
 		}
 		else
 			return BRASERO_BURN_NOT_SUPPORTED;
