@@ -7,19 +7,22 @@
  ***************************************************************************/
 
 /*
- *  This program is free software; you can redistribute it and/or modify
+ *  Brasero is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version  of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Brasero is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite , Boston, MA 111-17, USA.
+ *  along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.t, Fifth Floor
+ * 	Boston, MA  02110-1301, USA., Boston, MA 111-17, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -147,7 +150,7 @@ brasero_growisofs_read_stderr (BraseroProcess *process, const gchar *line)
 			/* NOTE: this has to be a multiple of 2048 */
 			brasero_job_set_output_size_for_current_track (BRASERO_JOB (process),
 								       sectors,
-								       sectors * 2048);
+								       sectors * 2048ULL);
 
 			/* we better tell growisofs to stop here as it returns 
 			 * a value of 1 when mkisofs is run with --print-size */
