@@ -7,19 +7,21 @@
  ***************************************************************************/
 
 /*
- *  This program is free software; you can redistribute it and/or modify
+ *  Brasero is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Brasero is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -243,11 +245,11 @@ brasero_transcode_create_volume (BraseroTranscode *transcode,
 	||  brasero_track_tag_lookup (track, BRASERO_TRACK_GAIN_VALUE, NULL) == BRASERO_BURN_OK) {
 		BRASERO_JOB_LOG (transcode, "Found audio levels tags");
 		volume = gst_element_factory_make ("rgvolume", NULL);
-		g_object_set (volume,
-			      "album-mode", FALSE,
-			      NULL);
-
-		if (!volume)
+		if (volume)
+			g_object_set (volume,
+				      "album-mode", FALSE,
+				      NULL);
+		else
 			BRASERO_JOB_LOG (transcode, "rgvolume object couldn't be created");
 	}
 
@@ -585,7 +587,10 @@ brasero_transcode_create_sibling_image (BraseroTranscode *transcode,
 	}
 
 	dest = brasero_track_new (BRASERO_TRACK_TYPE_AUDIO);
-	brasero_track_set_audio_source (dest, path_dest, BRASERO_AUDIO_FORMAT_RAW);
+	brasero_track_set_audio_source (dest,
+					path_dest,
+					BRASERO_AUDIO_FORMAT_RAW|
+					BRASERO_AUDIO_FORMAT_44100);
 
 	/* NOTE: there is no gap and start = 0 since these tracks are the result
 	 * of the transformation of previous ones */
@@ -860,7 +865,10 @@ brasero_transcode_push_track (BraseroTranscode *transcode)
 	brasero_job_get_output_type (BRASERO_JOB (transcode), &type);
 	track = brasero_track_new (BRASERO_TRACK_TYPE_AUDIO);
 
-	brasero_track_set_audio_source (track, output, BRASERO_AUDIO_FORMAT_RAW);
+	brasero_track_set_audio_source (track,
+					output,
+					BRASERO_AUDIO_FORMAT_RAW|
+					BRASERO_AUDIO_FORMAT_44100);
 	brasero_track_set_audio_boundaries (track, 0, length, 0);
 	brasero_track_set_audio_info (track, info);
 
@@ -1011,7 +1019,7 @@ brasero_transcode_pad_file (BraseroTranscode *transcode, GError **error)
 
 	output = NULL;
 	brasero_job_get_audio_output (BRASERO_JOB (transcode), &output);
-	fd = open (output, O_WRONLY | O_CREAT | O_APPEND, S_IRWXU | S_IRWXG | S_IROTH);
+	fd = open (output, O_WRONLY | O_CREAT | O_APPEND, S_IRWXU | S_IRGRP | S_IROTH);
 	g_free (output);
 
 	if (fd == -1) {
@@ -1438,7 +1446,8 @@ brasero_transcode_export_caps (BraseroPlugin *plugin, gchar **error)
 
 	output = brasero_caps_audio_new (BRASERO_PLUGIN_IO_ACCEPT_FILE|
 					 BRASERO_PLUGIN_IO_ACCEPT_PIPE,
-					 BRASERO_AUDIO_FORMAT_RAW);
+					 BRASERO_AUDIO_FORMAT_RAW|
+					 BRASERO_AUDIO_FORMAT_44100);
 
 	input = brasero_caps_audio_new (BRASERO_PLUGIN_IO_ACCEPT_FILE,
 					BRASERO_AUDIO_FORMAT_UNDEFINED);
