@@ -2,17 +2,17 @@
  *            burn-spc1.h
  *
  *  Fri Oct 20 13:18:29 2006
- *  Copyright  2006  algernon
- *  <algernon@localhost.localdomain>
+ *  Copyright  2006  Rouquier Philippe
+ *  <bonfire-app@wanadoo.fr>
  ****************************************************************************/
 
 /*
- * This program is free software; you can redistribute it and/or modify
+ * Brasero is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  * 
- * This program is distributed in the hope that it will be useful,
+ * Brasero is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU Library General Public License for more details.
@@ -41,6 +41,12 @@ brasero_spc1_mode_sense_get_page (BraseroDeviceHandle *handle,
 				  int *data_size,
 				  BraseroScsiErrCode *error);
 
+BraseroScsiResult
+brasero_spc1_mode_select (BraseroDeviceHandle *handle,
+			  BraseroScsiModeData *data,
+			  int size,
+			  BraseroScsiErrCode *error);
+
 G_END_DECLS
 
 #endif /* _BURN_SPC1_H */
